@@ -136,25 +136,6 @@ typedef enum
   CLUTTER_ABSOLUTE_ORIGIN_CHANGED = 1 << 1
 } ClutterAllocationFlags;
 
-/**
- * ClutterRedrawFlags:
- * @CLUTTER_REDRAW_CLIPPED_TO_BOX: Tells clutter the redraw is clipped
- *   to a given clip box in actor coordinates.
- * @CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION: Tells clutter the maximum
- *   extents of what needs to be redrawn lies within the actors
- *   current allocation.
- *
- * Flags passed to the clutter_actor_queue_redraw_with_clip ()
- * function
- *
- * Since: 1.2
- */
-typedef enum
-{
-  CLUTTER_REDRAW_CLIPPED_TO_BOX         = 0,
-  CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION  = 1 << 1
-} ClutterRedrawFlags;
-
 /**
  * ClutterActor:
  * @flags: #ClutterActorFlags
@@ -210,8 +191,6 @@ struct _ClutterActor
  * @apply_transform: virtual function, used when applying the transformations
  *   to an actor before painting it or when transforming coordinates or
  *   the allocation; it must chain up to the parent's implementation
- * @get_accessible: virtual function, returns the accessible object that
- *   describes the actor to an assistive technology.
  * @parent_set: signal class handler for the #ClutterActor::parent-set
  * @destroy: signal class handler for #ClutterActor::destroy
  * @pick: virtual function, used to draw an outline of the actor with
@@ -229,9 +208,13 @@ struct _ClutterActor
  * @enter_event: signal class closure for #ClutterActor::enter-event
  * @leave_event: signal class closure for #ClutterActor::leave-event
  * @captured_event: signal class closure for #ClutterActor::captured-event
- * @key_focus_in: signal class closure for #ClutterActor::focus-in
- * @key_focus_out: signal class closure for #ClutterActor::focus-out
+ * @key_focus_in: signal class closure for #ClutterActor::key-focus-in
+ * @key_focus_out: signal class closure for #ClutterActor::key-focus-out
  * @queue_relayout: class handler for #ClutterActor::queue-relayout
+ * @get_accessible: virtual function, returns the accessible object that
+ *   describes the actor to an assistive technology.
+ * @get_paint_volume: virtual function, for sub-classes to define their
+ *   #ClutterPaintVolume
  *
  * Base class for actors.
  */
@@ -306,9 +289,12 @@ struct _ClutterActorClass
   /* accessibility support */
   AtkObject * (* get_accessible)    (ClutterActor         *actor);
 
+  gboolean    (* get_paint_volume)  (ClutterActor         *actor,
+                                     ClutterPaintVolume   *volume);
+
   /*< private >*/
   /* padding for future expansion */
-  gpointer _padding_dummy[30];
+  gpointer _padding_dummy[29];
 };
 
 GType                 clutter_actor_get_type                  (void) G_GNUC_CONST;
@@ -361,6 +347,13 @@ void                  clutter_actor_allocate_available_size   (ClutterActor
                                                                gfloat                 available_width,
                                                                gfloat                 available_height,
                                                                ClutterAllocationFlags flags);
+void                  clutter_actor_allocate_align_fill       (ClutterActor          *self,
+                                                               const ClutterActorBox *box,
+                                                               gdouble                x_align,
+                                                               gdouble                y_align,
+                                                               gboolean               x_fill,
+                                                               gboolean               y_fill,
+                                                               ClutterAllocationFlags flags);
 void                  clutter_actor_get_allocation_box        (ClutterActor          *self,
                                                                ClutterActorBox       *box);
 void                  clutter_actor_get_allocation_geometry   (ClutterActor          *self,
@@ -578,6 +571,15 @@ gboolean             clutter_actor_has_allocation     (ClutterActor         *sel
 
 AtkObject *          clutter_actor_get_accessible     (ClutterActor         *self);
 
+gboolean             clutter_actor_has_key_focus      (ClutterActor         *self);
+
+const ClutterPaintVolume  *clutter_actor_get_paint_volume             (ClutterActor         *self);
+const ClutterPaintVolume  *clutter_actor_get_transformed_paint_volume (ClutterActor *self,
+                                                                       ClutterActor *relative_to_ancestor);
+
+gboolean             clutter_actor_get_paint_box      (ClutterActor         *self,
+                                                       ClutterActorBox      *box);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_ACTOR_H__ */
