@@ -55,6 +55,7 @@ struct _ClutterAlignConstraint
 {
   ClutterConstraint parent_instance;
 
+  ClutterActor *actor;
   ClutterActor *source;
   ClutterAlignAxis align_axis;
   gfloat factor;
@@ -71,52 +72,25 @@ enum
 
   PROP_SOURCE,
   PROP_ALIGN_AXIS,
-  PROP_FACTOR
+  PROP_FACTOR,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_TYPE (ClutterAlignConstraint,
                clutter_align_constraint,
                CLUTTER_TYPE_CONSTRAINT);
 
 static void
-update_actor_position (ClutterAlignConstraint *align)
-{
-  gfloat source_width, source_height;
-  gfloat actor_width, actor_height;
-  ClutterActor *actor;
-
-  if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (align)))
-    return;
-
-  actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (align));
-  if (actor == NULL)
-    return;
-
-  clutter_actor_get_size (align->source, &source_width, &source_height);
-  clutter_actor_get_size (actor, &actor_width, &actor_height);
-
-  switch (align->align_axis)
-    {
-    case CLUTTER_ALIGN_X_AXIS:
-      clutter_actor_set_x (actor, (source_width - actor_width) * align->factor);
-      break;
-
-    case CLUTTER_ALIGN_Y_AXIS:
-      clutter_actor_set_y (actor, (source_height - actor_height) * align->factor);
-      break;
-    }
-}
-
-static void
-source_position_changed (GObject                *gobject,
-                         GParamSpec             *pspec,
+source_position_changed (ClutterActor           *actor,
+                         const ClutterActorBox  *allocation,
+                         ClutterAllocationFlags  flags,
                          ClutterAlignConstraint *align)
 {
-  if (strcmp (pspec->name, "width") == 0 ||
-      strcmp (pspec->name, "height") == 0)
-    {
-      update_actor_position (align);
-    }
+  if (align->actor != NULL)
+    clutter_actor_queue_relayout (align->actor);
 }
 
 static void
@@ -127,57 +101,59 @@ source_destroyed (ClutterActor           *actor,
 }
 
 static void
-_clutter_align_constraint_set_source (ClutterAlignConstraint *align,
-                                      ClutterActor           *source)
+clutter_align_constraint_set_actor (ClutterActorMeta *meta,
+                                    ClutterActor     *new_actor)
 {
-  ClutterActor *old_source = align->source;
+  ClutterAlignConstraint *align = CLUTTER_ALIGN_CONSTRAINT (meta);
+  ClutterActorMetaClass *parent;
 
-  if (old_source != NULL)
-    {
-      g_signal_handlers_disconnect_by_func (old_source,
-                                            G_CALLBACK (source_destroyed),
-                                            align);
-      g_signal_handlers_disconnect_by_func (old_source,
-                                            G_CALLBACK (source_position_changed),
-                                            align);
-    }
+  /* store the pointer to the actor, for later use */
+  align->actor = new_actor;
 
-  align->source = source;
-  g_signal_connect (align->source, "notify",
-                    G_CALLBACK (source_position_changed),
-                    align);
-  g_signal_connect (align->source, "destroy",
-                    G_CALLBACK (source_destroyed),
-                    align);
-
-  update_actor_position (align);
-
-  g_object_notify (G_OBJECT (align), "source");
+  parent = CLUTTER_ACTOR_META_CLASS (clutter_align_constraint_parent_class);
+  parent->set_actor (meta, new_actor);
 }
 
 static void
-_clutter_align_constraint_set_align_axis (ClutterAlignConstraint *align,
-                                          ClutterAlignAxis        axis)
+clutter_align_constraint_update_allocation (ClutterConstraint *constraint,
+                                            ClutterActor      *actor,
+                                            ClutterActorBox   *allocation)
 {
-  if (align->align_axis == axis)
+  ClutterAlignConstraint *align = CLUTTER_ALIGN_CONSTRAINT (constraint);
+  ClutterActorMeta *meta = CLUTTER_ACTOR_META (constraint);
+  gfloat source_width, source_height;
+  gfloat actor_width, actor_height;
+  gfloat source_x, source_y;
+
+  if (align->source == NULL)
     return;
 
-  align->align_axis = axis;
+  if (!clutter_actor_meta_get_enabled (meta))
+    return;
 
-  update_actor_position (align);
+  clutter_actor_get_position (align->source, &source_x, &source_y);
+  clutter_actor_get_size (align->source, &source_width, &source_height);
 
-  g_object_notify (G_OBJECT (align), "align-axis");
-}
+  switch (align->align_axis)
+    {
+    case CLUTTER_ALIGN_X_AXIS:
+      actor_width = clutter_actor_box_get_width (allocation);
+      allocation->x1 = ((source_width - actor_width) * align->factor)
+                     + source_x;
+      allocation->x2 = allocation->x1 + actor_width;
+      break;
 
-static void
-_clutter_align_constraint_set_factor (ClutterAlignConstraint *align,
-                                      gfloat                  factor)
-{
-  align->factor = CLAMP (factor, 0.0, 1.0);
+    case CLUTTER_ALIGN_Y_AXIS:
+      actor_height = clutter_actor_box_get_height (allocation);
+      allocation->y1 = ((source_height - actor_height) * align->factor)
+                     + source_y;
+      allocation->y2 = allocation->y1 + actor_height;
+      break;
 
-  update_actor_position (align);
-
-  g_object_notify (G_OBJECT (align), "factor");
+    default:
+      g_assert_not_reached ();
+      break;
+    }
 }
 
 static void
@@ -191,15 +167,15 @@ clutter_align_constraint_set_property (GObject      *gobject,
   switch (prop_id)
     {
     case PROP_SOURCE:
-      _clutter_align_constraint_set_source (align, g_value_get_object (value));
+      clutter_align_constraint_set_source (align, g_value_get_object (value));
       break;
 
     case PROP_ALIGN_AXIS:
-      _clutter_align_constraint_set_align_axis (align, g_value_get_enum (value));
+      clutter_align_constraint_set_align_axis (align, g_value_get_enum (value));
       break;
 
     case PROP_FACTOR:
-      _clutter_align_constraint_set_factor (align, g_value_get_float (value));
+      clutter_align_constraint_set_factor (align, g_value_get_float (value));
       break;
 
     default:
@@ -240,11 +216,17 @@ static void
 clutter_align_constraint_class_init (ClutterAlignConstraintClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
+  ClutterConstraintClass *constraint_class = CLUTTER_CONSTRAINT_CLASS (klass);
   GParamSpec *pspec;
 
   gobject_class->set_property = clutter_align_constraint_set_property;
   gobject_class->get_property = clutter_align_constraint_get_property;
 
+  meta_class->set_actor = clutter_align_constraint_set_actor;
+
+  constraint_class->update_allocation = clutter_align_constraint_update_allocation;
+
   /**
    * ClutterAlignConstraint:source:
    *
@@ -253,11 +235,12 @@ clutter_align_constraint_class_init (ClutterAlignConstraintClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_object ("source",
-                               "Source",
-                               "The source of the alignment",
+                               P_("Source"),
+                               P_("The source of the alignment"),
                                CLUTTER_TYPE_ACTOR,
                                CLUTTER_PARAM_READWRITE |
                                G_PARAM_CONSTRUCT);
+  obj_props[PROP_SOURCE] = pspec;
   g_object_class_install_property (gobject_class, PROP_SOURCE, pspec);
 
   /**
@@ -268,12 +251,13 @@ clutter_align_constraint_class_init (ClutterAlignConstraintClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_enum ("align-axis",
-                             "Align Axis",
-                             "The axis to align the position to",
+                             P_("Align Axis"),
+                             P_("The axis to align the position to"),
                              CLUTTER_TYPE_ALIGN_AXIS,
                              CLUTTER_ALIGN_X_AXIS,
                              CLUTTER_PARAM_READWRITE |
                              G_PARAM_CONSTRUCT);
+  obj_props[PROP_ALIGN_AXIS] = pspec;
   g_object_class_install_property (gobject_class, PROP_ALIGN_AXIS, pspec);
 
   /**
@@ -281,26 +265,28 @@ clutter_align_constraint_class_init (ClutterAlignConstraintClass *klass)
    *
    * The alignment factor, as a normalized value between 0.0 and 1.0
    *
-   * The #ClutterAlignConstraint:factor depends on the
-   * #ClutterAlignConstraint:align-axis value: with %CLUTTER_ALIGN_X_AXIS,
-   * 0.0 means left and 1.0 means right; with %CLUTTER_ALIGN_Y_AXIS, 0.0
-   * means top and 1.0 means bottom
+   * The factor depends on the #ClutterAlignConstraint:align-axis property:
+   * with an align-axis value of %CLUTTER_ALIGN_X_AXIS, 0.0 means left and
+   * 1.0 means right; with a value of %CLUTTER_ALIGN_Y_AXIS, 0.0 means top
+   * and 1.0 means bottom.
    *
    * Since: 1.4
    */
   pspec = g_param_spec_float ("factor",
-                              "Factor",
-                              "The alignment factor, between 0.0 and 1.0",
+                              P_("Factor"),
+                              P_("The alignment factor, between 0.0 and 1.0"),
                               0.0, 1.0,
                               0.0,
                               CLUTTER_PARAM_READWRITE |
                               G_PARAM_CONSTRUCT);
+  obj_props[PROP_FACTOR] = pspec;
   g_object_class_install_property (gobject_class, PROP_FACTOR, pspec);
 }
 
 static void
 clutter_align_constraint_init (ClutterAlignConstraint *self)
 {
+  self->actor = NULL;
   self->source = NULL;
   self->align_axis = CLUTTER_ALIGN_X_AXIS;
   self->factor = 0.0f;
@@ -308,13 +294,14 @@ clutter_align_constraint_init (ClutterAlignConstraint *self)
 
 /**
  * clutter_align_constraint_new:
- * @source: the #ClutterActor to use as the source of the alignment
+ * @source: (allow-none): the #ClutterActor to use as the source of the
+ *   alignment, or %NULL
  * @axis: the axis to be used to compute the alignment
  * @factor: the alignment factor, between 0.0 and 1.0
  *
  * Creates a new constraint, aligning a #ClutterActor's position with
- * regards of the size of the actor to @source, with the given alignment
- * @factor
+ * regards of the size of the actor to @source, with the given
+ * alignment @factor
  *
  * Return value: the newly created #ClutterAlignConstraint
  *
@@ -325,7 +312,7 @@ clutter_align_constraint_new (ClutterActor     *source,
                               ClutterAlignAxis  axis,
                               gfloat            factor)
 {
-  g_return_val_if_fail (CLUTTER_IS_ACTOR (source), NULL);
+  g_return_val_if_fail (source == NULL || CLUTTER_IS_ACTOR (source), NULL);
 
   return g_object_new (CLUTTER_TYPE_ALIGN_CONSTRAINT,
                        "source", source,
@@ -333,3 +320,167 @@ clutter_align_constraint_new (ClutterActor     *source,
                        "factor", factor,
                        NULL);
 }
+
+/**
+ * clutter_align_constraint_set_source:
+ * @align: a #ClutterAlignConstraint
+ * @source: (allow-none): a #ClutterActor, or %NULL to unset the source
+ *
+ * Sets the source of the alignment constraint
+ *
+ * Since: 1.4
+ */
+void
+clutter_align_constraint_set_source (ClutterAlignConstraint *align,
+                                     ClutterActor           *source)
+{
+  ClutterActor *old_source;
+
+  g_return_if_fail (CLUTTER_IS_ALIGN_CONSTRAINT (align));
+  g_return_if_fail (source == NULL || CLUTTER_IS_ACTOR (source));
+
+  if (align->source == source)
+    return;
+
+  old_source = align->source;
+  if (old_source != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (old_source,
+                                            G_CALLBACK (source_destroyed),
+                                            align);
+      g_signal_handlers_disconnect_by_func (old_source,
+                                            G_CALLBACK (source_position_changed),
+                                            align);
+    }
+
+  align->source = source;
+  if (align->source != NULL)
+    {
+      g_signal_connect (align->source, "allocation-changed",
+                        G_CALLBACK (source_position_changed),
+                        align);
+      g_signal_connect (align->source, "destroy",
+                        G_CALLBACK (source_destroyed),
+                        align);
+
+      if (align->actor != NULL)
+        clutter_actor_queue_relayout (align->actor);
+    }
+
+  _clutter_notify_by_pspec (G_OBJECT (align), obj_props[PROP_SOURCE]);
+}
+
+/**
+ * clutter_align_constraint_get_source:
+ * @align: a #ClutterAlignConstraint
+ *
+ * Retrieves the source of the alignment
+ *
+ * Return value: (transfer none): the #ClutterActor used as the source
+ *   of the alignment
+ *
+ * Since: 1.4
+ */
+ClutterActor *
+clutter_align_constraint_get_source (ClutterAlignConstraint *align)
+{
+  g_return_val_if_fail (CLUTTER_IS_ALIGN_CONSTRAINT (align), NULL);
+
+  return align->source;
+}
+
+/**
+ * clutter_align_constraint_set_align_axis:
+ * @align: a #ClutterAlignConstraint
+ * @axis: the axis to which the alignment refers to
+ *
+ * Sets the axis to which the alignment refers to
+ *
+ * Since: 1.4
+ */
+void
+clutter_align_constraint_set_align_axis (ClutterAlignConstraint *align,
+                                         ClutterAlignAxis        axis)
+{
+  g_return_if_fail (CLUTTER_IS_ALIGN_CONSTRAINT (align));
+
+  if (align->align_axis == axis)
+    return;
+
+  align->align_axis = axis;
+
+  if (align->actor != NULL)
+    clutter_actor_queue_relayout (align->actor);
+
+  _clutter_notify_by_pspec (G_OBJECT (align), obj_props[PROP_ALIGN_AXIS]);
+}
+
+/**
+ * clutter_align_constraint_get_align_axis:
+ * @align: a #ClutterAlignConstraint
+ *
+ * Retrieves the value set using clutter_align_constraint_set_align_axis()
+ *
+ * Return value: the alignment axis
+ *
+ * Since: 1.4
+ */
+ClutterAlignAxis
+clutter_align_constraint_get_align_axis (ClutterAlignConstraint *align)
+{
+  g_return_val_if_fail (CLUTTER_IS_ALIGN_CONSTRAINT (align),
+                        CLUTTER_ALIGN_X_AXIS);
+
+  return align->align_axis;
+}
+
+/**
+ * clutter_align_constraint_set_factor:
+ * @align: a #ClutterAlignConstraint
+ * @factor: the alignment factor, between 0.0 and 1.0
+ *
+ * Sets the alignment factor of the constraint
+ *
+ * The factor depends on the #ClutterAlignConstraint:align-axis property
+ * and it is a value between 0.0 (meaning left, when
+ * #ClutterAlignConstraint:align-axis is set to %CLUTTER_ALIGN_X_AXIS; or
+ * meaning top, when #ClutterAlignConstraint:align-axis is set to
+ * %CLUTTER_ALIGN_Y_AXIS) and 1.0 (meaning right, when
+ * #ClutterAlignConstraint:align-axis is set to %CLUTTER_ALIGN_X_AXIS; or
+ * meaning bottom, when #ClutterAlignConstraint:align-axis is set to
+ * %CLUTTER_ALIGN_Y_AXIS). A value of 0.5 aligns in the middle in either
+ * cases
+ *
+ * Since: 1.4
+ */
+void
+clutter_align_constraint_set_factor (ClutterAlignConstraint *align,
+                                     gfloat                  factor)
+{
+  g_return_if_fail (CLUTTER_IS_ALIGN_CONSTRAINT (align));
+
+  align->factor = CLAMP (factor, 0.0, 1.0);
+
+  if (align->actor != NULL)
+    clutter_actor_queue_relayout (align->actor);
+
+  _clutter_notify_by_pspec (G_OBJECT (align), obj_props[PROP_FACTOR]);
+}
+
+/**
+ * clutter_align_constraint_get_factor:
+ * @align: a #ClutterAlignConstraint
+ *
+ * Retrieves the factor set using clutter_align_constraint_set_factor()
+ *
+ * Return value: the alignment factor
+ *
+ * Since: 1.4
+ */
+gfloat
+clutter_align_constraint_get_factor (ClutterAlignConstraint *align)
+{
+  g_return_val_if_fail (CLUTTER_IS_ALIGN_CONSTRAINT (align), 0.0);
+
+  return align->factor;
+}
