@@ -276,6 +276,7 @@ property_iter_new (ClutterAnimator *animator,
                    PropObjectKey   *key,
                    GType            type)
 {
+  ClutterAnimatorPrivate *priv = animator->priv;
   PropertyIter *property_iter = g_slice_new (PropertyIter);
   ClutterInterval *interval = g_object_new (CLUTTER_TYPE_INTERVAL,
                                             "value-type", type,
@@ -287,8 +288,7 @@ property_iter_new (ClutterAnimator *animator,
   property_iter->interval = interval;
   property_iter->key = key;
   property_iter->alpha = clutter_alpha_new ();
-  clutter_alpha_set_timeline (property_iter->alpha,
-                              animator->priv->slave_timeline);
+  clutter_alpha_set_timeline (property_iter->alpha, priv->slave_timeline);
 
   /* as well as the alpha */
   g_object_ref_sink (property_iter->alpha);
@@ -434,6 +434,18 @@ clutter_animator_key_free (gpointer boxed)
   g_slice_free (ClutterAnimatorKey, key);
 }
 
+static void
+clutter_animator_dispose (GObject *object)
+{
+  ClutterAnimator *animator = CLUTTER_ANIMATOR (object);
+  ClutterAnimatorPrivate *priv = animator->priv;
+
+  clutter_animator_set_timeline (animator, NULL);
+  g_object_unref (priv->slave_timeline);
+
+  G_OBJECT_CLASS (clutter_animator_parent_class)->dispose (object);
+}
+
 static void
 clutter_animator_finalize (GObject *object)
 {
@@ -444,9 +456,6 @@ clutter_animator_finalize (GObject *object)
   g_list_free (priv->score);
   priv->score = NULL;
 
-  g_object_unref (priv->timeline);
-  g_object_unref (priv->slave_timeline);
-
   g_hash_table_destroy (priv->properties);
 
   G_OBJECT_CLASS (clutter_animator_parent_class)->finalize (object);
@@ -852,6 +861,7 @@ clutter_animator_compute_value (ClutterAnimator *animator,
                                 gdouble          progress,
                                 GValue          *value)
 {
+  ClutterAnimatorPrivate *priv;
   ClutterAnimatorKey   key;
   ClutterAnimatorKey  *previous;
   ClutterAnimatorKey  *next = NULL;
@@ -861,12 +871,14 @@ clutter_animator_compute_value (ClutterAnimator *animator,
   GList               *next_l;
   gboolean             ease_in;
   ClutterInterpolation interpolation;
- 
+
   g_return_val_if_fail (CLUTTER_IS_ANIMATOR (animator), FALSE);
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (property_name, FALSE);
   g_return_val_if_fail (value, FALSE);
-  
+
+  priv = animator->priv;
+
   ease_in = clutter_animator_property_get_ease_in (animator, object,
                                                    property_name);
   interpolation = clutter_animator_property_get_interpolation (animator,
@@ -882,7 +894,7 @@ clutter_animator_compute_value (ClutterAnimator *animator,
 
   initial_l = g_list_find_custom (animator->priv->score, &key,
                                   sort_actor_prop_func);
-  if (!initial_l)
+  if (initial_l == NULL)
     return FALSE;
 
   /* first find the interval we belong in, that is the first interval
@@ -905,9 +917,7 @@ clutter_animator_compute_value (ClutterAnimator *animator,
              }
          }
        else
-         {
-           next = NULL;
-         }
+         next = NULL;
 
        if (progress < previous->progress)
          {
@@ -924,13 +934,14 @@ clutter_animator_compute_value (ClutterAnimator *animator,
             g_value_copy (&previous->value, value);
             return TRUE;
          }
+
        if (next && next->progress >= progress)
          {
             ClutterInterval *interval;
             ClutterAlpha    *alpha;
 
             gdouble sub_progress = (progress - previous->progress)
-                                    / (next->progress - previous->progress);
+                                 / (next->progress - previous->progress);
             /* this should be our interval */
             interval = g_object_new (CLUTTER_TYPE_INTERVAL,
                                      "value-type", pspec->value_type,
@@ -945,17 +956,15 @@ clutter_animator_compute_value (ClutterAnimator *animator,
                 g_value_unset (&tmp_value);
               }
             else
-              {
-                clutter_interval_set_initial_value (interval, &previous->value);
-              }
+              clutter_interval_set_initial_value (interval, &previous->value);
+
             clutter_interval_set_final_value (interval, &next->value);
 
             alpha = clutter_alpha_new ();
-            clutter_alpha_set_timeline (alpha,
-                                        animator->priv->slave_timeline);
+            clutter_alpha_set_timeline (alpha, priv->slave_timeline);
             clutter_alpha_set_mode (alpha, next->mode);
 
-            clutter_timeline_advance (animator->priv->slave_timeline,
+            clutter_timeline_advance (priv->slave_timeline,
                                       sub_progress * 10000);
             sub_progress = clutter_alpha_get_alpha (alpha);
 
@@ -1049,7 +1058,7 @@ clutter_animator_set_timeline (ClutterAnimator *animator,
   priv->timeline = timeline;
   if (timeline != NULL)
     {
-      g_object_ref_sink (priv->timeline);
+      g_object_ref (priv->timeline);
 
       g_signal_connect (priv->timeline, "new-frame",
                         G_CALLBACK (animation_animator_new_frame),
@@ -1445,23 +1454,14 @@ again:
         }
     }
 
-  if (object)
-    {
-      GHashTableIter iter;
-      gpointer key, value;
+  /* clear off cached state for all properties, this is regenerated in a
+   * correct state by animation_animator_started
+   */
+  g_hash_table_remove_all (priv->properties);
 
-again2:
-      g_hash_table_iter_init (&iter, priv->properties);
-      while (g_hash_table_iter_next (&iter, &key, &value))
-        {
-          PropObjectKey *prop_actor_key = key;
-          if (prop_actor_key->object == object)
-            {
-              g_hash_table_remove (priv->properties, key);
-              goto again2;
-            }
-        }
-    }
+  /* if the animator is already running reinitialize internal iterators */
+  if (clutter_timeline_is_playing (priv->timeline))
+    animation_animator_started (priv->timeline, animator);
 }
 
 /**
@@ -1773,6 +1773,7 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
 
   gobject_class->set_property = clutter_animator_set_property;
   gobject_class->get_property = clutter_animator_get_property;
+  gobject_class->dispose = clutter_animator_dispose;
   gobject_class->finalize = clutter_animator_finalize;
 
   /**
@@ -1813,6 +1814,7 @@ static void
 clutter_animator_init (ClutterAnimator *animator)
 {
   ClutterAnimatorPrivate *priv;
+  ClutterTimeline *timeline;
 
   animator->priv = priv = CLUTTER_ANIMATOR_GET_PRIVATE (animator);
 
@@ -1821,7 +1823,9 @@ clutter_animator_init (ClutterAnimator *animator)
                                             prop_actor_key_free,
                                             property_iter_free);
 
-  clutter_animator_set_timeline (animator, clutter_timeline_new (2000));
+  timeline = clutter_timeline_new (2000);
+  clutter_animator_set_timeline (animator, timeline);
+  g_object_unref (timeline);
 
   priv->slave_timeline = clutter_timeline_new (10000);
 }
