@@ -74,8 +74,12 @@ enum
   PROP_ANGLE_TILT_Y,
   PROP_ANGLE_TILT_Z,
   PROP_DIRECTION,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 struct _ClutterBehaviourEllipsePrivate
 {
   ClutterKnot  center;
@@ -391,6 +395,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_START] = pspec;
   g_object_class_install_property (object_class, PROP_ANGLE_START, pspec);
 
   /**
@@ -406,6 +411,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_END] = pspec;
   g_object_class_install_property (object_class, PROP_ANGLE_END, pspec);
 
   /**
@@ -421,6 +427,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                                0.0, 360.0,
                                360.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_TILT_X] = pspec;
   g_object_class_install_property (object_class, PROP_ANGLE_TILT_X, pspec);
 
   /**
@@ -436,6 +443,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                                0.0, 360.0,
                                360.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_TILT_Y] = pspec;
   g_object_class_install_property (object_class, PROP_ANGLE_TILT_Y, pspec);
 
   /**
@@ -451,6 +459,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                                0.0, 360.0,
                                360.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_TILT_Z] = pspec;
   g_object_class_install_property (object_class, PROP_ANGLE_TILT_Z, pspec);
 
   /**
@@ -466,6 +475,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                             0, G_MAXINT,
                             100,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_WIDTH] = pspec;
   g_object_class_install_property (object_class, PROP_WIDTH, pspec);
 
   /**
@@ -481,6 +491,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                             0, G_MAXINT,
                             50,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_HEIGHT] = pspec;
   g_object_class_install_property (object_class, PROP_HEIGHT, pspec);
 
   /**
@@ -495,6 +506,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                               P_("Center of ellipse"),
                               CLUTTER_TYPE_KNOT,
                               CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER] = pspec;
   g_object_class_install_property (object_class, PROP_CENTER, pspec);
 
   /**
@@ -510,6 +522,7 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
                              CLUTTER_TYPE_ROTATE_DIRECTION,
                              CLUTTER_ROTATE_CW,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DIRECTION] = pspec;
   g_object_class_install_property (object_class, PROP_DIRECTION, pspec);
 }
 
@@ -535,7 +548,7 @@ clutter_behaviour_ellipse_init (ClutterBehaviourEllipse * self)
 
 /**
  * clutter_behaviour_ellipse_new:
- * @alpha: a #ClutterAlpha, or %NULL
+ * @alpha: (allow-none): a #ClutterAlpha instance, or %NULL
  * @x: x coordinace of the center
  * @y: y coordiance of the center
  * @width: width of the ellipse
@@ -551,6 +564,10 @@ clutter_behaviour_ellipse_init (ClutterBehaviourEllipse * self)
  * interval <0, 360); if @start is equal to @end, the behaviour will
  * rotate by exacly 360 degrees.
  *
+ * If @alpha is not %NULL, the #ClutterBehaviour will take ownership
+ * of the #ClutterAlpha instance. In the case when @alpha is %NULL,
+ * it can be set later with clutter_behaviour_set_alpha().
+ *
  * Return value: the newly created #ClutterBehaviourEllipse
  *
  * Since: 0.4
@@ -609,15 +626,15 @@ clutter_behaviour_ellipse_set_center (ClutterBehaviourEllipse *self,
       priv->center.x = x;
       priv->center.y = y;
 
-      g_object_notify (G_OBJECT (self), "center");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_CENTER]);
     }
 }
 
 /**
  * clutter_behaviour_ellipse_get_center:
  * @self: a #ClutterBehaviourEllipse
- * @x: return location for the X coordinate of the center, or %NULL
- * @y: return location for the Y coordinate of the center, or %NULL
+ * @x: (out): return location for the X coordinate of the center, or %NULL
+ * @y: (out): return location for the Y coordinate of the center, or %NULL
  *
  * Gets the center of the elliptical path path.
  *
@@ -665,7 +682,7 @@ clutter_behaviour_ellipse_set_width (ClutterBehaviourEllipse *self,
     {
       priv->a = width / 2;
 
-      g_object_notify (G_OBJECT (self), "width");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_WIDTH]);
     }
 }
 
@@ -710,7 +727,7 @@ clutter_behaviour_ellipse_set_height (ClutterBehaviourEllipse *self,
     {
       priv->b = height / 2;
 
-      g_object_notify (G_OBJECT (self), "height");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_HEIGHT]);
     }
 }
 
@@ -758,7 +775,7 @@ clutter_behaviour_ellipse_set_angle_start (ClutterBehaviourEllipse *self,
   if (priv->angle_start != new_angle)
     {
       priv->angle_start = new_angle;
-      g_object_notify (G_OBJECT (self), "angle-start");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_START]);
     }
 }
 
@@ -807,7 +824,7 @@ clutter_behaviour_ellipse_set_angle_end (ClutterBehaviourEllipse *self,
     {
       priv->angle_end = new_angle;
 
-      g_object_notify (G_OBJECT (self), "angle-end");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_END]);
     }
 }
 
@@ -858,7 +875,7 @@ clutter_behaviour_ellipse_set_angle_tilt (ClutterBehaviourEllipse *self,
         {
           priv->angle_tilt_x = angle_tilt;
 
-          g_object_notify (G_OBJECT (self), "angle-tilt-x");
+          _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_TILT_X]);
         }
       break;
 
@@ -867,7 +884,7 @@ clutter_behaviour_ellipse_set_angle_tilt (ClutterBehaviourEllipse *self,
         {
           priv->angle_tilt_y = angle_tilt;
 
-          g_object_notify (G_OBJECT (self), "angle-tilt-y");
+          _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_TILT_Y]);
         }
       break;
 
@@ -876,7 +893,7 @@ clutter_behaviour_ellipse_set_angle_tilt (ClutterBehaviourEllipse *self,
         {
           priv->angle_tilt_z = angle_tilt;
 
-          g_object_notify (G_OBJECT (self), "angle-tilt-z");
+          _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_TILT_Z]);
         }
       break;
     }
@@ -943,21 +960,21 @@ clutter_behaviour_ellipse_set_tilt (ClutterBehaviourEllipse *self,
     {
       priv->angle_tilt_x = angle_tilt_x;
 
-      g_object_notify (G_OBJECT (self), "angle-tilt-x");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_TILT_X]);
     }
 
   if (priv->angle_tilt_y != angle_tilt_y)
     {
       priv->angle_tilt_y = angle_tilt_y;
 
-      g_object_notify (G_OBJECT (self), "angle-tilt-y");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_TILT_Y]);
     }
 
   if (priv->angle_tilt_z != angle_tilt_z)
     {
       priv->angle_tilt_z = angle_tilt_z;
 
-      g_object_notify (G_OBJECT (self), "angle-tilt-z");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ANGLE_TILT_Z]);
     }
 
   g_object_thaw_notify (G_OBJECT (self));
@@ -966,9 +983,9 @@ clutter_behaviour_ellipse_set_tilt (ClutterBehaviourEllipse *self,
 /**
  * clutter_behaviour_ellipse_get_tilt:
  * @self: a #ClutterBehaviourEllipse
- * @angle_tilt_x: return location for tilt angle on the X axis, or %NULL.
- * @angle_tilt_y: return location for tilt angle on the Y axis, or %NULL.
- * @angle_tilt_z: return location for tilt angle on the Z axis, or %NULL.
+ * @angle_tilt_x: (out): return location for tilt angle on the X axis, or %NULL.
+ * @angle_tilt_y: (out): return location for tilt angle on the Y axis, or %NULL.
+ * @angle_tilt_z: (out): return location for tilt angle on the Z axis, or %NULL.
  *
  * Gets the tilt of the ellipse around the center in Y axis.
  *
@@ -1038,6 +1055,6 @@ clutter_behaviour_ellipse_set_direction (ClutterBehaviourEllipse *self,
     {
       priv->direction = direction;
 
-      g_object_notify (G_OBJECT (self), "direction");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_DIRECTION]);
     }
 }
