@@ -140,9 +140,13 @@ enum
   PROP_0,
 
   PROP_X_ALIGN,
-  PROP_Y_ALIGN
+  PROP_Y_ALIGN,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_TYPE (ClutterBinLayer,
                clutter_bin_layer,
                CLUTTER_TYPE_LAYOUT_META);
@@ -171,7 +175,7 @@ set_layer_x_align (ClutterBinLayer     *self,
   manager = clutter_layout_meta_get_manager (meta);
   clutter_layout_manager_layout_changed (manager);
 
-  g_object_notify (G_OBJECT (self), "x-align");
+  _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_X_ALIGN]);
 }
 
 static void
@@ -190,7 +194,7 @@ set_layer_y_align (ClutterBinLayer     *self,
   manager = clutter_layout_meta_get_manager (meta);
   clutter_layout_manager_layout_changed (manager);
 
-  g_object_notify (G_OBJECT (self), "y-align");
+  _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_Y_ALIGN]);
 }
 
 static void
@@ -251,23 +255,25 @@ clutter_bin_layer_class_init (ClutterBinLayerClass *klass)
   gobject_class->get_property = clutter_bin_layer_get_property;
 
   pspec = g_param_spec_enum ("x-align",
-                             "Horizontal Alignment",
-                             "Horizontal alignment for the actor "
-                             "inside the layer",
+                             P_("Horizontal Alignment"),
+                             P_("Horizontal alignment for the actor "
+                                "inside the layer"),
                              CLUTTER_TYPE_BIN_ALIGNMENT,
                              CLUTTER_BIN_ALIGNMENT_CENTER,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_LAYER_X_ALIGN] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_LAYER_X_ALIGN,
                                    pspec);
 
   pspec = g_param_spec_enum ("y-align",
-                             "Vertical Alignment",
-                             "Vertical alignment for the actor "
-                             "inside the layer manager",
+                             P_("Vertical Alignment"),
+                             P_("Vertical alignment for the actor "
+                                "inside the layer manager"),
                              CLUTTER_TYPE_BIN_ALIGNMENT,
                              CLUTTER_BIN_ALIGNMENT_CENTER,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_LAYER_Y_ALIGN] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_LAYER_Y_ALIGN,
                                    pspec);
@@ -299,7 +305,7 @@ set_x_align (ClutterBinLayout    *self,
       manager = CLUTTER_LAYOUT_MANAGER (self);
       clutter_layout_manager_layout_changed (manager);
 
-      g_object_notify (G_OBJECT (self), "x-align");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_X_ALIGN]);
     }
 }
 
@@ -318,7 +324,7 @@ set_y_align (ClutterBinLayout    *self,
       manager = CLUTTER_LAYOUT_MANAGER (self);
       clutter_layout_manager_layout_changed (manager);
 
-      g_object_notify (G_OBJECT (self), "y-align");
+      _clutter_notify_by_pspec (G_OBJECT (self), obj_props[PROP_Y_ALIGN]);
     }
 }
 
@@ -429,102 +435,36 @@ clutter_bin_layout_allocate (ClutterLayoutManager   *manager,
       ClutterLayoutMeta *meta;
       ClutterBinLayer *layer;
       ClutterActorBox child_alloc = { 0, };
-      gfloat child_width, child_height;
-      ClutterRequestMode request;
+      gdouble x_align, y_align;
+      gboolean x_fill, y_fill;
 
       meta = clutter_layout_manager_get_child_meta (manager,
                                                     container,
                                                     child);
       layer = CLUTTER_BIN_LAYER (meta);
 
-      if (layer->x_align == CLUTTER_BIN_ALIGNMENT_FILL)
-        {
-          child_alloc.x1 = 0;
-          child_alloc.x2 = ceilf (available_w);
-        }
-
-      if (layer->y_align == CLUTTER_BIN_ALIGNMENT_FILL)
-        {
-          child_alloc.y1 = 0;
-          child_alloc.y2 = ceilf (available_h);
-        }
-
-      /* if we are filling horizontally and vertically then we
-       * can break here because we already have a full allocation
-       */
-      if (layer->x_align == CLUTTER_BIN_ALIGNMENT_FILL &&
-          layer->y_align == CLUTTER_BIN_ALIGNMENT_FILL)
-        {
-          clutter_actor_allocate (child, &child_alloc, flags);
-          continue;
-        }
-
-      child_width = child_height = 0;
-      request = clutter_actor_get_request_mode (child);
-      if (request == CLUTTER_REQUEST_HEIGHT_FOR_WIDTH)
-        {
-          gfloat min_width, nat_width;
-          gfloat min_height, nat_height;
-
-          clutter_actor_get_preferred_width (child, available_h,
-                                             &min_width,
-                                             &nat_width);
-          child_width = CLAMP (nat_width, min_width, available_w);
-
-          clutter_actor_get_preferred_height (child, child_width,
-                                              &min_height,
-                                              &nat_height);
-          child_height = CLAMP (nat_height, min_height, available_h);
-        }
-      else if (request == CLUTTER_REQUEST_WIDTH_FOR_HEIGHT)
-        {
-          gfloat min_width, nat_width;
-          gfloat min_height, nat_height;
-
-          clutter_actor_get_preferred_height (child, available_w,
-                                              &min_height,
-                                              &nat_height);
-          child_height = CLAMP (nat_height, min_height, available_h);
-
-          clutter_actor_get_preferred_width (child, child_height,
-                                             &min_width,
-                                             &nat_width);
-          child_width = CLAMP (nat_width, min_width, available_w);
-        }
-
       if (layer->x_align == CLUTTER_BIN_ALIGNMENT_FIXED)
-        {
-          child_alloc.x1 = ceilf (clutter_actor_get_x (child));
-          child_alloc.x2 = ceilf (child_alloc.x1 + child_width);
-        }
+        child_alloc.x1 = clutter_actor_get_x (child);
       else
-        {
-          gdouble x_align = get_bin_alignment_factor (layer->x_align);
-
-          if (layer->x_align != CLUTTER_BIN_ALIGNMENT_FILL)
-            {
-              child_alloc.x1 = ceilf ((available_w - child_width) * x_align);
-              child_alloc.x2 = ceilf (child_alloc.x1 + child_width);
-            }
-        }
+        child_alloc.x1 = 0.0f;
 
       if (layer->y_align == CLUTTER_BIN_ALIGNMENT_FIXED)
-        {
-          child_alloc.y1 = ceilf (clutter_actor_get_y (child));
-          child_alloc.y2 = ceilf (child_alloc.y1 + child_height);
-        }
+        child_alloc.y1 = clutter_actor_get_y (child);
       else
-        {
-          gdouble y_align = get_bin_alignment_factor (layer->y_align);
+        child_alloc.y1 = 0.0f;
 
-          if (layer->y_align != CLUTTER_BIN_ALIGNMENT_FILL)
-            {
-              child_alloc.y1 = ceilf ((available_h - child_height) * y_align);
-              child_alloc.y2 = ceilf (child_alloc.y1 + child_height);
-            }
-        }
+      child_alloc.x2 = available_w;
+      child_alloc.y2 = available_h;
 
-      clutter_actor_allocate (child, &child_alloc, flags);
+      x_fill = (layer->x_align == CLUTTER_BIN_ALIGNMENT_FILL);
+      y_fill = (layer->y_align == CLUTTER_BIN_ALIGNMENT_FILL);
+      x_align = get_bin_alignment_factor (layer->x_align);
+      y_align = get_bin_alignment_factor (layer->y_align);
+
+      clutter_actor_allocate_align_fill (child, &child_alloc,
+                                         x_align, y_align,
+                                         x_fill, y_fill,
+                                         flags);
     }
 
   g_list_free (children);
@@ -640,12 +580,13 @@ clutter_bin_layout_class_init (ClutterBinLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_enum ("x-align",
-                             "Horizontal Alignment",
-                             "Default horizontal alignment for the actors "
-                             "inside the layout manager",
+                             P_("Horizontal Alignment"),
+                             P_("Default horizontal alignment for the actors "
+                                "inside the layout manager"),
                              CLUTTER_TYPE_BIN_ALIGNMENT,
                              CLUTTER_BIN_ALIGNMENT_CENTER,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_X_ALIGN] = pspec;
   g_object_class_install_property (gobject_class, PROP_X_ALIGN, pspec);
 
   /**
@@ -657,12 +598,13 @@ clutter_bin_layout_class_init (ClutterBinLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_enum ("y-align",
-                             "Vertical Alignment",
-                             "Default vertical alignment for the actors "
-                             "inside the layout manager",
+                             P_("Vertical Alignment"),
+                             P_("Default vertical alignment for the actors "
+                                "inside the layout manager"),
                              CLUTTER_TYPE_BIN_ALIGNMENT,
                              CLUTTER_BIN_ALIGNMENT_CENTER,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_Y_ALIGN] = pspec;
   g_object_class_install_property (gobject_class, PROP_Y_ALIGN, pspec);
 
   layout_class->get_preferred_width =
