@@ -51,7 +51,6 @@ typedef struct _ClutterBindConstraint   ClutterBindConstraint;
  * ClutterBindCoordinate:
  * @CLUTTER_BIND_X: Bind the X coordinate
  * @CLUTTER_BIND_Y: Bind the Y coordinate
- * @CLUTTER_BIND_Z: Bind the Z coordinate
  * @CLUTTER_BIND_WIDTH: Bind the width
  * @CLUTTER_BIND_HEIGHT: Bidng the height
  *
@@ -62,7 +61,6 @@ typedef struct _ClutterBindConstraint   ClutterBindConstraint;
 typedef enum { /*< prefix=CLUTTER_BIND >*/
   CLUTTER_BIND_X,
   CLUTTER_BIND_Y,
-  CLUTTER_BIND_Z,
   CLUTTER_BIND_WIDTH,
   CLUTTER_BIND_HEIGHT
 } ClutterBindCoordinate;
