@@ -92,6 +92,9 @@ static void              create_child_meta  (ClutterContainer *container,
                                              ClutterActor     *actor);
 static void              destroy_child_meta (ClutterContainer *container,
                                              ClutterActor     *actor);
+static void              child_notify       (ClutterContainer *container,
+                                             ClutterActor     *child,
+                                             GParamSpec       *pspec);
 
 static void
 clutter_container_base_init (gpointer g_iface)
@@ -150,7 +153,8 @@ clutter_container_base_init (gpointer g_iface)
       /**
        * ClutterContainer::child-notify:
        * @container: the container which received the signal
-       * @actor: the child that has had a property set. 
+       * @actor: the child that has had a property set
+       * @pspec: the #GParamSpec of the property set
        *
        * The ::child-notify signal is emitted each time a property is
        * being set through the clutter_container_child_set() and
@@ -161,10 +165,10 @@ clutter_container_base_init (gpointer g_iface)
       container_signals[CHILD_NOTIFY] =
         g_signal_new (I_("child-notify"),
                       iface_type,
-                      G_SIGNAL_RUN_FIRST,
+                      G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
                       G_STRUCT_OFFSET (ClutterContainerIface, child_notify),
                       NULL, NULL,
-                      _clutter_marshal_VOID__OBJECT_OBJECT_PARAM,
+                      _clutter_marshal_VOID__OBJECT_PARAM,
                       G_TYPE_NONE, 2,
                       CLUTTER_TYPE_ACTOR, G_TYPE_PARAM);
 
@@ -172,6 +176,7 @@ clutter_container_base_init (gpointer g_iface)
       iface->create_child_meta  = create_child_meta;
       iface->destroy_child_meta = destroy_child_meta;
       iface->get_child_meta     = get_child_meta;
+      iface->child_notify       = child_notify;
     }
 }
 
@@ -922,6 +927,13 @@ clutter_container_class_list_child_properties (GObjectClass *klass,
   return retval;
 }
 
+static void
+child_notify (ClutterContainer *container,
+              ClutterActor     *actor,
+              GParamSpec       *pspec)
+{
+}
+
 static inline void
 container_set_child_property (ClutterContainer *container,
                               ClutterActor     *actor,
@@ -929,14 +941,15 @@ container_set_child_property (ClutterContainer *container,
                               GParamSpec       *pspec)
 {
   ClutterChildMeta *data;
-  ClutterContainerIface *iface;
 
   data = clutter_container_get_child_meta (container, actor);
   g_object_set_property (G_OBJECT (data), pspec->name, value);
 
-  iface = CLUTTER_CONTAINER_GET_IFACE (container);
-  if (G_LIKELY (iface->child_notify))
-    iface->child_notify (container, actor, pspec);
+  g_signal_emit (container, container_signals[CHILD_NOTIFY],
+                 (pspec->flags & G_PARAM_STATIC_NAME)
+                   ? g_quark_from_static_string (pspec->name)
+                   : g_quark_from_string (pspec->name),
+                 actor, pspec);
 }
 
 /**
@@ -1210,3 +1223,31 @@ clutter_container_child_get (ClutterContainer *container,
 
   va_end (var_args);
 }
+
+/**
+ * clutter_container_child_notify:
+ * @container: a #ClutterContainer
+ * @child: a #ClutterActor
+ * @pspec: a #GParamSpec
+ *
+ * Calls the <function>child_notify()</function> virtual function of
+ * #ClutterContainer. The default implementation will emit the
+ * #ClutterContainer::child-notify signal.
+ *
+ * Since: 1.6
+ */
+void
+clutter_container_child_notify (ClutterContainer *container,
+                                ClutterActor     *child,
+                                GParamSpec       *pspec)
+{
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+  g_return_if_fail (CLUTTER_IS_ACTOR (child));
+  g_return_if_fail (pspec != NULL);
+
+  g_return_if_fail (clutter_actor_get_parent (child) == CLUTTER_ACTOR (container));
+
+  CLUTTER_CONTAINER_GET_IFACE (container)->child_notify (container,
+                                                         child,
+                                                         pspec);
+}
