@@ -72,7 +72,8 @@ struct _ClutterDragActionPrivate
 {
   ClutterActor *stage;
 
-  gfloat drag_threshold;
+  gfloat x_drag_threshold;
+  gfloat y_drag_threshold;
   ClutterActor *drag_handle;
   ClutterDragAxis drag_axis;
 
@@ -98,11 +99,16 @@ enum
 {
   PROP_0,
 
-  PROP_DRAG_THRESHOLD,
+  PROP_X_DRAG_THRESHOLD,
+  PROP_Y_DRAG_THRESHOLD,
   PROP_DRAG_HANDLE,
-  PROP_DRAG_AXIS
+  PROP_DRAG_AXIS,
+
+  PROP_LAST
 };
 
+static GParamSpec *drag_props[PROP_LAST] = { NULL, };
+
 enum
 {
   DRAG_BEGIN,
@@ -184,8 +190,8 @@ emit_drag_motion (ClutterDragAction *action,
 
   if (priv->emit_delayed_press)
     {
-      if (ABS (delta_x) >= priv->drag_threshold ||
-          ABS (delta_y) >= priv->drag_threshold)
+      if (ABS (delta_x) >= priv->x_drag_threshold ||
+          ABS (delta_y) >= priv->y_drag_threshold)
         {
           priv->emit_delayed_press = FALSE;
 
@@ -263,6 +269,12 @@ on_captured_event (ClutterActor      *stage,
         emit_drag_end (action, actor, event);
       break;
 
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
+      if (priv->in_drag)
+        return TRUE;
+      break;
+
     default:
       break;
     }
@@ -298,7 +310,7 @@ on_button_press (ClutterActor      *actor,
                                        &priv->transformed_press_x,
                                        &priv->transformed_press_y);
 
-  if (priv->drag_threshold == 0)
+  if (priv->x_drag_threshold == 0 || priv->y_drag_threshold == 0)
     emit_drag_begin (action, actor, event);
   else
     priv->emit_delayed_press = TRUE;
@@ -365,11 +377,18 @@ clutter_drag_action_set_property (GObject      *gobject,
                                   GParamSpec   *pspec)
 {
   ClutterDragAction *action = CLUTTER_DRAG_ACTION (gobject);
+  ClutterDragActionPrivate *priv = action->priv;
 
   switch (prop_id)
     {
-    case PROP_DRAG_THRESHOLD:
-      clutter_drag_action_set_drag_threshold (action, g_value_get_uint (value));
+    case PROP_X_DRAG_THRESHOLD:
+      clutter_drag_action_set_drag_threshold (action, g_value_get_uint (value),
+                                              priv->y_drag_threshold);
+      break;
+
+    case PROP_Y_DRAG_THRESHOLD:
+      clutter_drag_action_set_drag_threshold (action, priv->x_drag_threshold,
+                                              g_value_get_uint (value));
       break;
 
     case PROP_DRAG_HANDLE:
@@ -395,8 +414,12 @@ clutter_drag_action_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_DRAG_THRESHOLD:
-      g_value_set_uint (value, priv->drag_threshold);
+    case PROP_X_DRAG_THRESHOLD:
+      g_value_set_uint (value, priv->x_drag_threshold);
+      break;
+
+    case PROP_Y_DRAG_THRESHOLD:
+      g_value_set_uint (value, priv->y_drag_threshold);
       break;
 
     case PROP_DRAG_HANDLE:
@@ -443,7 +466,6 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
 {
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterDragActionPrivate));
 
@@ -456,30 +478,51 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
   klass->drag_motion = clutter_drag_action_real_drag_motion;
 
   /**
-   * ClutterDragAction:drag-threshold:
+   * ClutterDragAction:x-drag-threshold:
    *
-   * The threshold, in pixels, that begins a drag action
+   * The horizontal threshold, in pixels, that begins a drag action
    *
    * When set to a non-zero value, #ClutterDragAction will only emit
-   * #ClutterDragAction::drag-begin if the pointer has moved at least
-   * of the given amount of pixels since the button press event
+   * #ClutterDragAction::drag-begin if the pointer has moved
+   * horizontally at least of the given amount of pixels since
+   * the button press event
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_uint ("drag-threshold",
-                             P_("Drag Threshold"),
-                             P_("The amount of pixels required to start dragging"),
-                             0, G_MAXUINT,
-                             0,
-                             CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_DRAG_THRESHOLD, pspec);
+  drag_props[PROP_X_DRAG_THRESHOLD] =
+    g_param_spec_uint ("x-drag-threshold",
+                       P_("Horizontal Drag Threshold"),
+                       P_("The horizontal amount of pixels required to start dragging"),
+                       0, G_MAXUINT,
+                       0,
+                       CLUTTER_PARAM_READWRITE);
+
+  /**
+   * ClutterDragAction:y-drag-threshold:
+   *
+   * The vertical threshold, in pixels, that begins a drag action
+   *
+   * When set to a non-zero value, #ClutterDragAction will only emit
+   * #ClutterDragAction::drag-begin if the pointer has moved
+   * vertically at least of the given amount of pixels since
+   * the button press event
+   *
+   * Since: 1.4
+   */
+  drag_props[PROP_Y_DRAG_THRESHOLD] =
+    g_param_spec_uint ("y-drag-threshold",
+                       P_("Vertical Drag Threshold"),
+                       P_("The vertical amount of pixels required to start dragging"),
+                       0, G_MAXUINT,
+                       0,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterDragAction:drag-handle:
    *
    * The #ClutterActor that is effectively being dragged
    *
-   * A #ClutterDragActor will, be default, use the #ClutterActor that
+   * A #ClutterDragAction will, be default, use the #ClutterActor that
    * has been attached to the action; it is possible to create a
    * separate #ClutterActor and use it instead.
    *
@@ -488,12 +531,12 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_object ("drag-handle",
-                               P_("Drag Handle"),
-                               P_("The actor that is being dragged"),
-                               CLUTTER_TYPE_ACTOR,
-                               CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_DRAG_HANDLE, pspec);
+  drag_props[PROP_DRAG_HANDLE] =
+    g_param_spec_object ("drag-handle",
+                         P_("Drag Handle"),
+                         P_("The actor that is being dragged"),
+                         CLUTTER_TYPE_ACTOR,
+                         CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterDragAction:drag-axis:
@@ -502,13 +545,15 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_enum ("drag-axis",
-                             P_("Drag Axis"),
-                             P_("Constraints the dragging to an axis"),
-                             CLUTTER_TYPE_DRAG_AXIS,
-                             CLUTTER_DRAG_AXIS_NONE,
-                             CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_DRAG_AXIS, pspec);
+  drag_props[PROP_DRAG_AXIS] =
+    g_param_spec_enum ("drag-axis",
+                       P_("Drag Axis"),
+                       P_("Constraints the dragging to an axis"),
+                       CLUTTER_TYPE_DRAG_AXIS,
+                       CLUTTER_DRAG_AXIS_NONE,
+                       CLUTTER_PARAM_READWRITE);
+
+  _clutter_object_class_install_properties (klass, PROP_LAST, drag_props);
 
   /**
    * ClutterDragAction::drag-begin:
@@ -522,7 +567,8 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * starts the dragging
    *
    * The emission of this signal can be delayed by using the
-   * #ClutterDragAction:drag-threshold property
+   * #ClutterDragAction:x-drag-threshold and
+   * #ClutterDragAction:y-drag-threshold properties
    *
    * Since: 1.4
    */
@@ -635,47 +681,70 @@ clutter_drag_action_new (void)
 /**
  * clutter_drag_action_set_drag_threshold:
  * @action: a #ClutterDragAction
- * @threshold: a distance, in pixels
+ * @x_threshold: a distance on the horizontal axis, in pixels
+ * @y_threshold: a distance on the vertical axis, in pixels
  *
- * Sets the drag threshold that must be cleared by the pointer
- * before @action can begin the dragging
+ * Sets the horizontal and vertical drag thresholds that must be
+ * cleared by the pointer before @action can begin the dragging
  *
  * Since: 1.4
  */
 void
 clutter_drag_action_set_drag_threshold (ClutterDragAction *action,
-                                        guint              threshold)
+                                        guint              x_threshold,
+					guint              y_threshold)
 {
   ClutterDragActionPrivate *priv;
+  GObject *self;
 
   g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
 
+  self = G_OBJECT (action);
   priv = action->priv;
 
-  if (priv->drag_threshold == threshold)
-    return;
+  g_object_freeze_notify (self);
 
-  priv->drag_threshold = threshold;
+  if (priv->x_drag_threshold != x_threshold)
+    {
+      priv->x_drag_threshold = x_threshold;
 
-  g_object_notify (G_OBJECT (action), "drag-threshold");
+      _clutter_notify_by_pspec (self, drag_props[PROP_X_DRAG_THRESHOLD]);
+    }
+
+  if (priv->y_drag_threshold != y_threshold)
+    {
+      priv->y_drag_threshold = y_threshold;
+
+      _clutter_notify_by_pspec (self, drag_props[PROP_Y_DRAG_THRESHOLD]);
+    }
+
+  g_object_thaw_notify (self);
 }
 
 /**
  * clutter_drag_action_get_drag_threshold:
  * @action: a #ClutterDragAction
+ * @x_threshold: (out): return location for the horizontal drag
+ *   threshold value, in pixels
+ * @y_threshold: (out): return location for the vertical drag
+ *   threshold value, in pixels
  *
- * Retrieves the value set by clutter_drag_action_set_drag_threshold()
- *
- * Return value: the drag threshold value, in pixels
+ * Retrieves the values set by clutter_drag_action_set_drag_threshold()
  *
  * Since: 1.4
  */
-guint
-clutter_drag_action_get_drag_threshold (ClutterDragAction *action)
+void
+clutter_drag_action_get_drag_threshold (ClutterDragAction *action,
+                                        guint             *x_threshold,
+					guint             *y_threshold)
 {
-  g_return_val_if_fail (CLUTTER_IS_DRAG_ACTION (action), 0);
+  g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
 
-  return action->priv->drag_threshold;
+  if (x_threshold)
+    *x_threshold = action->priv->x_drag_threshold;
+
+  if (y_threshold)
+    *y_threshold = action->priv->y_drag_threshold;
 }
 
 /**
@@ -703,7 +772,7 @@ clutter_drag_action_set_drag_handle (ClutterDragAction *action,
 
   priv->drag_handle = handle;
 
-  g_object_notify (G_OBJECT (action), "drag-handle");
+  _clutter_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_HANDLE]);
 }
 
 /**
@@ -751,7 +820,7 @@ clutter_drag_action_set_drag_axis (ClutterDragAction *action,
 
   priv->drag_axis = axis;
 
-  g_object_notify (G_OBJECT (action), "drag-axis");
+  _clutter_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_AXIS]);
 }
 
 /**
