@@ -75,9 +75,13 @@ enum
 {
   PROP_0,
 
-  PROP_VALUE_TYPE
+  PROP_VALUE_TYPE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   INITIAL = 0,
@@ -409,6 +413,7 @@ clutter_interval_class_init (ClutterIntervalClass *klass)
                               P_("The type of the values in the interval"),
                               G_TYPE_NONE,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+  obj_props[PROP_VALUE_TYPE] = pspec;
   g_object_class_install_property (gobject_class, PROP_VALUE_TYPE, pspec);
 }
 
@@ -592,7 +597,7 @@ clutter_interval_new_with_values (GType         gtype,
  *
  * Creates a copy of @interval.
  *
- * Return value: the newly created #ClutterInterval
+ * Return value: (transfer full): the newly created #ClutterInterval
  *
  * Since: 1.0
  */
@@ -693,7 +698,7 @@ clutter_interval_set_initial_value (ClutterInterval *interval,
 /**
  * clutter_interval_get_initial_value:
  * @interval: a #ClutterInterval
- * @value: a #GValue
+ * @value: (out caller-allocates): a #GValue
  *
  * Retrieves the initial value of @interval and copies
  * it into @value.
@@ -762,7 +767,7 @@ clutter_interval_set_final_value (ClutterInterval *interval,
 /**
  * clutter_interval_get_final_value:
  * @interval: a #ClutterInterval
- * @value: a #GValue
+ * @value: (out caller-allocates): a #GValue
  *
  * Retrieves the final value of @interval and copies
  * it into @value.
@@ -896,7 +901,7 @@ clutter_interval_validate (ClutterInterval *interval,
  * clutter_interval_compute_value:
  * @interval: a #ClutterInterval
  * @factor: the progress factor, between 0 and 1
- * @value: return location for an initialized #GValue
+ * @value: (out caller-allocates): return location for an initialized #GValue
  *
  * Computes the value between the @interval boundaries given the
  * progress @factor and copies it into @value.
@@ -963,7 +968,7 @@ clutter_interval_compute (ClutterInterval *interval,
 }
 
 /**
- * clutter_interval_register_progress_func:
+ * clutter_interval_register_progress_func: (skip)
  * @value_type: a #GType
  * @func: a #ClutterProgressFunc, or %NULL to unset a previously
  *   set progress function
