@@ -193,7 +193,16 @@ struct _ClutterModelPrivate
 {
   GType                  *column_types;
   gchar                 **column_names;
-  guint                   n_columns;
+
+  /* we use an integer here because we want to be able to use -1 as a
+   * guard value, to allow calling set_names() and set_types() from
+   * sub-classes of ClutterModel. see bug:
+   *
+   *   http://bugzilla.openedhand.com/show_bug.cgi?id=2032
+   *
+   * for a reference.
+   */
+  gint                    n_columns; 
 
   ClutterModelFilterFunc  filter_func;
   gpointer                filter_data;
@@ -235,7 +244,12 @@ clutter_model_real_get_column_name (ClutterModel *model,
 static guint
 clutter_model_real_get_n_columns (ClutterModel *model)
 {
-  return model->priv->n_columns;
+  ClutterModelPrivate *priv = model->priv;
+
+  if (priv->n_columns < 0)
+    return 0;
+
+  return priv->n_columns;
 }
 
 static void 
@@ -335,7 +349,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, row_added),
                   NULL, NULL,
-                  clutter_marshal_VOID__OBJECT,
+                  _clutter_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CLUTTER_TYPE_MODEL_ITER);
    /**
@@ -355,7 +369,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, row_removed),
                   NULL, NULL,
-                  clutter_marshal_VOID__OBJECT,
+                  _clutter_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CLUTTER_TYPE_MODEL_ITER);
    /**
@@ -375,7 +389,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, row_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__OBJECT,
+                  _clutter_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CLUTTER_TYPE_MODEL_ITER);
   /**
@@ -392,7 +406,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, sort_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
    /**
    * ClutterModel::filter-changed:
@@ -408,7 +422,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, filter_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 }
 
@@ -419,7 +433,7 @@ clutter_model_init (ClutterModel *self)
   
   self->priv = priv = CLUTTER_MODEL_GET_PRIVATE (self);
 
-  priv->n_columns = 0;
+  priv->n_columns = -1;
   priv->column_types = NULL;
   priv->column_names = NULL;
 
@@ -1447,7 +1461,7 @@ clutter_model_get_sorting_column (ClutterModel *model)
 /**
  * clutter_model_foreach:
  * @model: a #ClutterModel
- * @func: a #ClutterModelForeachFunc
+ * @func: (scope call): a #ClutterModelForeachFunc
  * @user_data: user data to pass to @func
  *
  * Calls @func for each row in the model. 
@@ -2248,7 +2262,7 @@ clutter_model_iter_get_row (ClutterModelIter *iter)
  *
  * Copies the passed iterator.
  *
- * Return value: a copy of the iterator, or %NULL
+ * Return value: (transfer full): a copy of the iterator, or %NULL
  *
  * Since: 0.8
  */
