@@ -91,6 +91,23 @@ typedef enum {
   CLUTTER_INTERNAL_CHILD = 1 << 6
 } ClutterPrivateFlags;
 
+/*
+ * ClutterRedrawFlags:
+ * @CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION: Tells clutter the maximum
+ *   extents of what needs to be redrawn lies within the actors
+ *   current allocation. (Only use this for 2D actors though because
+ *   any actor with depth may be projected outside of its allocation)
+ *
+ * Flags passed to the clutter_actor_queue_redraw_with_clip ()
+ * function
+ *
+ * Since: 1.6
+ */
+typedef enum
+{
+  CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION  = 1 << 0
+} ClutterRedrawFlags;
+
 struct _ClutterInputDevice
 {
   GObject parent_instance;
@@ -183,11 +200,108 @@ struct _ClutterMainContext
   GList *repaint_funcs;
 
   ClutterSettings *settings;
-
-  gint picks_per_frame;
-  gboolean have_complete_pick_buffer;
 };
 
+struct _ClutterPaintVolume
+{
+  ClutterActor *actor;
+
+  /* cuboid for the volume:
+   *
+   *       4━━━━━━━┓5
+   *    ┏━━━━━━━━┓╱┃
+   *    ┃0 ┊7   1┃ ┃
+   *    ┃   ┄┄┄┄┄┃┄┃6
+   *    ┃3      2┃╱
+   *    ┗━━━━━━━━┛
+   *
+   *   0: top, left (origin)  : always valid
+   *   1: top, right          : always valid
+   *   2: bottom, right       :  updated lazily
+   *   3: bottom, left        : always valid
+   *
+   *   4: top, left, back     : always valid
+   *   5: top, right, back    :  updated lazily
+   *   6: bottom, right, back :  updated lazily
+   *   7: bottom, left, back  :  updated lazily
+   *
+   * Elements 0, 1, 3 and 4 are filled in by the PaintVolume setters
+   *
+   * Note: the reason for this ordering is that we can simply ignore
+   * elements 4, 5, 6 and 7 most of the time for 2D actors when
+   * calculating the projected paint box.
+   */
+  ClutterVertex vertices[8];
+
+  /* As an optimization for internally managed PaintVolumes we allow
+   * initializing ClutterPaintVolume variables allocated on the stack
+   * so we can avoid hammering the slice allocator. */
+  guint is_static:1;
+
+  /* A newly initialized PaintVolume is considered empty as it is
+   * degenerate on all three axis.
+   *
+   * We consider this carefully when we union an empty volume with
+   * another so that the union simply results in a copy of the other
+   * volume instead of also bounding the origin of the empty volume.
+   *
+   * For example this is a convenient property when calculating the
+   * volume of a container as the union of the volume of its children
+   * where the initial volume passed to the containers
+   * ->get_paint_volume method will be empty. */
+  guint is_empty:1;
+
+  /* TRUE when we've updated the values we calculate lazily */
+  guint is_complete:1;
+
+  /* TRUE if vertices 4-7 can be ignored. (Only valid if complete is
+   * TRUE) */
+  guint is_2d:1;
+
+  /* Set to TRUE initialy but cleared if the paint volume is
+   * transfomed by a matrix. */
+  guint is_axis_aligned:1;
+
+
+  /* Note: There is a precedence to the above bitfields that should be
+   * considered whenever we implement code that manipulates
+   * PaintVolumes...
+   *
+   * Firstly if ->is_empty == TRUE then the values for ->is_complete
+   * and ->is_2d are undefined, so you should typically check
+   * ->is_empty as the first priority.
+   *
+   * XXX: document other invariables...
+   */
+};
+
+/* ClutterActorTraverseFlags:
+ *
+ * Controls some options for how clutter_actor_traverse() iterates
+ * through the graph.
+ */
+typedef enum _ClutterActorTraverseFlags
+{
+  CLUTTER_ACTOR_TRAVERSE_PLACE_HOLDER  = 1L<<0
+} ClutterActorTraverseFlags;
+
+/* ClutterForeachCallback:
+ * @actor: The actor being iterated
+ * @user_data: The private data specified when starting the iteration
+ *
+ * A generic callback for iterating over actor, such as with
+ * _clutter_actor_foreach_child. The difference when compared to
+ * #ClutterCallback is that it returns a boolean so it is possible to break
+ * out of an iteration early.
+ *
+ * Return value: %TRUE to continue iterating or %FALSE to break iteration
+ * early.
+ */
+typedef gboolean (*ClutterForeachCallback) (ClutterActor *actor,
+                                            void *user_data);
+
+typedef struct _ClutterStageQueueRedrawEntry ClutterStageQueueRedrawEntry;
+
 #define CLUTTER_CONTEXT()	(_clutter_context_get_default ())
 ClutterMainContext *_clutter_context_get_default (void);
 gboolean            _clutter_context_is_initialized (void);
@@ -242,6 +356,8 @@ void _clutter_stage_manager_set_default_stage (ClutterStageManager *stage_manage
                                                ClutterStage        *stage);
 
 /* stage */
+void                _clutter_stage_do_paint             (ClutterStage *stage,
+                                                         const ClutterGeometry *clip);
 void                _clutter_stage_set_window           (ClutterStage       *stage,
                                                          ClutterStageWindow *stage_window);
 ClutterStageWindow *_clutter_stage_get_window           (ClutterStage       *stage);
@@ -279,6 +395,26 @@ int      _clutter_stage_get_pending_swaps      (ClutterStage *stage);
 
 gboolean _clutter_stage_has_full_redraw_queued (ClutterStage *stage);
 
+void     _clutter_stage_set_pick_buffer_valid             (ClutterStage *stage,
+                                                           gboolean      valid);
+gboolean _clutter_stage_get_pick_buffer_valid             (ClutterStage *stage);
+void     _clutter_stage_increment_picks_per_frame_counter (ClutterStage *stage);
+void     _clutter_stage_reset_picks_per_frame_counter     (ClutterStage *stage);
+guint    _clutter_stage_get_picks_per_frame_counter       (ClutterStage *stage);
+
+ClutterPaintVolume *_clutter_stage_paint_volume_stack_allocate (ClutterStage *stage);
+void                _clutter_stage_paint_volume_stack_free_all (ClutterStage *stage);
+
+const ClutterGeometry *_clutter_stage_get_clip (ClutterStage *stage);
+
+ClutterStageQueueRedrawEntry *_clutter_stage_queue_actor_redraw           (ClutterStage *stage,
+                                                                           ClutterStageQueueRedrawEntry *entry,
+                                                                           ClutterActor *actor,
+                                                                           ClutterPaintVolume *clip);
+
+void                          _clutter_stage_queue_redraw_entry_invalidate (ClutterStageQueueRedrawEntry *entry);
+
+
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
 
@@ -341,6 +477,14 @@ gboolean _clutter_boolean_handled_accumulator (GSignalInvocationHint *ihint,
                                                const GValue          *handler_return,
                                                gpointer               dummy);
 
+gint          _clutter_actor_get_n_children             (ClutterActor *self);
+gboolean      _clutter_actor_foreach_child              (ClutterActor *self,
+                                                         ClutterForeachCallback callback,
+                                                         void *user_data);
+gboolean      _clutter_actor_traverse                   (ClutterActor *actor,
+                                                         ClutterActorTraverseFlags flags,
+                                                         ClutterForeachCallback callback,
+                                                         void *user_data);
 ClutterActor *_clutter_actor_get_stage_internal         (ClutterActor *actor);
 
 void _clutter_actor_apply_modelview_transform           (ClutterActor *self,
@@ -365,23 +509,27 @@ void _clutter_actor_set_enable_paint_unmapped (ClutterActor *self,
 void _clutter_actor_set_has_pointer (ClutterActor *self,
                                      gboolean      has_pointer);
 
-gboolean _clutter_actor_transform_and_project_box (ClutterActor          *self,
-                                                   const ClutterActorBox *box,
-                                                   ClutterVertex          verts[]);
-
-void _clutter_actor_queue_redraw_with_clip (ClutterActor          *self,
-                                            ClutterRedrawFlags     flags,
-                                            ClutterActorBox       *clip);
-const ClutterActorBox *_clutter_actor_get_queue_redraw_clip (ClutterActor *self);
-void _clutter_actor_set_queue_redraw_clip (ClutterActor *self,
-                                           const ClutterActorBox *clip);
+void _clutter_actor_queue_redraw_with_clip   (ClutterActor              *self,
+                                              ClutterRedrawFlags         flags,
+                                              ClutterPaintVolume        *clip_volume);
+const ClutterPaintVolume *_clutter_actor_get_queue_redraw_clip (ClutterActor *self);
+void _clutter_actor_set_queue_redraw_clip     (ClutterActor             *self,
+                                               const ClutterPaintVolume *clip_volume);
+void _clutter_actor_finish_queue_redraw       (ClutterActor             *self,
+                                               ClutterPaintVolume       *clip);
 
 void _clutter_run_repaint_functions (void);
 
 gint32 _clutter_backend_get_units_serial (ClutterBackend *backend);
 
-gboolean _clutter_effect_pre_paint  (ClutterEffect *effect);
-void     _clutter_effect_post_paint (ClutterEffect *effect);
+gboolean _clutter_effect_pre_paint        (ClutterEffect      *effect);
+void     _clutter_effect_post_paint       (ClutterEffect      *effect);
+gboolean _clutter_effect_get_paint_volume (ClutterEffect      *effect,
+                                           ClutterPaintVolume *volume);
+
+void _clutter_constraint_update_allocation (ClutterConstraint *constraint,
+                                            ClutterActor      *actor,
+                                            ClutterActorBox   *allocation);
 
 GType _clutter_layout_manager_get_child_meta_type (ClutterLayoutManager *manager);
 
@@ -401,6 +549,57 @@ gpointer _clutter_event_get_platform_data (const ClutterEvent *event);
 
 #endif
 
+/* wrapper for g_object_class_install_properties() */
+static inline void
+_clutter_object_class_install_properties (gpointer     oclass,
+                                          guint        n_pspecs,
+                                          GParamSpec **pspecs)
+{
+#if GLIB_CHECK_VERSION (2, 26, 0)
+  g_object_class_install_properties (oclass, n_pspecs, pspecs);
+#else
+  int i;
+
+  /* XXX - we start at 1 because 0 is a reserved property id */
+  for (i = 1; i < n_pspecs; i++)
+    g_object_class_install_property (oclass, i, pspecs[i]);
+#endif
+}
+
+void                _clutter_paint_volume_init_static          (ClutterActor *actor,
+                                                                ClutterPaintVolume *pv);
+ClutterPaintVolume *_clutter_paint_volume_new                  (ClutterActor       *actor);
+void                _clutter_paint_volume_copy_static          (const ClutterPaintVolume *src_pv,
+                                                                ClutterPaintVolume *dst_pv);
+void                _clutter_paint_volume_set_from_volume      (ClutterPaintVolume *pv,
+                                                                const ClutterPaintVolume *src);
+
+void                _clutter_paint_volume_complete             (ClutterPaintVolume *pv);
+void                _clutter_paint_volume_transform            (ClutterPaintVolume *pv,
+                                                                const CoglMatrix *matrix);
+void                _clutter_paint_volume_project              (ClutterPaintVolume *pv,
+                                                                const CoglMatrix   *modelview,
+                                                                const CoglMatrix   *projection,
+                                                                const int          *viewport);
+void                _clutter_paint_volume_get_bounding_box     (ClutterPaintVolume *pv,
+                                                                ClutterActorBox    *box);
+void                _clutter_paint_volume_axis_align           (ClutterPaintVolume *pv);
+void                _clutter_paint_volume_set_reference_actor  (ClutterPaintVolume *pv,
+                                                                ClutterActor *actor);
+
+
+
+void                _clutter_util_fully_transform_vertices (const CoglMatrix *modelview,
+                                                            const CoglMatrix *projection,
+                                                            const int *viewport,
+                                                            const ClutterVertex *vertices_in,
+                                                            ClutterVertex *vertices_out,
+                                                            int n_vertices);
+
+gboolean           _clutter_actor_set_default_paint_volume (ClutterActor *self,
+                                                            GType         check_gtype,
+                                                            ClutterPaintVolume *volume);
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_PRIVATE_H */
