@@ -269,6 +269,8 @@ clutter_state_remove_key_internal (ClutterState *this,
   if (source_state_name)
     source_state = clutter_state_fetch_state (this, source_state_name, FALSE);
 
+again_from_start:
+
   if (target_state_name != NULL)
     state_list = g_list_append (NULL, (gpointer) target_state_name);
   else
@@ -278,31 +280,36 @@ clutter_state_remove_key_internal (ClutterState *this,
     {
       State *target_state;
       target_state = clutter_state_fetch_state (this, s->data, FALSE);
-
+      /* Go through each TargetState */
       if (target_state)
         {
           GList *k;
-again:
+again_for_target_state:
           for (k = target_state->keys; k != NULL; k = k->next)
             {
               ClutterStateKey *key = k->data;
 
+              /* Check if each key matches query */
               if (   (object == NULL        || (object == key->object))
                   && (source_state == NULL  || (source_state == key->source_state))
                   && (property_name == NULL || ((property_name == key->property_name))))
                 {
+                  /* Remove matching key */
                   target_state->keys = g_list_remove (target_state->keys, key);
-
-                  if (target_state->keys == NULL)
-                    {
-                      /* no more keys, so remove this state */
-                      clutter_state_remove_key (this, s->data, NULL, NULL, NULL);
-                      g_hash_table_remove (this->priv->states, s->data);
-                    }
-
                   key->is_inert = is_inert;
                   clutter_state_key_free (key);
-                  goto again;
+
+                  /* no more keys with transitions to this target_state*/
+                  if (target_state->keys == NULL)
+                    {
+                      /* remove any keys that exist that uses this state as a source */
+                      clutter_state_remove_key (this, s->data, NULL, NULL, NULL);
+
+                      g_hash_table_remove (this->priv->states, s->data);
+                      goto again_from_start; /* we have just freed State *target_state, so
+                                                need to restart removal */
+                    }
+                  goto again_for_target_state;
                 }
             }
         }
@@ -503,6 +510,10 @@ clutter_state_change (ClutterState *state,
                                          priv->target_state_name);
   priv->target_state = new_state;
 
+  if (animator == NULL && new_state->keys == NULL)
+    animator = clutter_state_get_animator (state, NULL,
+                                           priv->target_state_name);
+
   if (animator != NULL)
     {
       /* we've got an animator overriding the tweened animation */
@@ -561,7 +572,9 @@ clutter_state_change (ClutterState *state,
  * The state will animate during its transition, see
  * #clutter_state_warp_to_state for animation-free state switching.
  *
- * Return value: the #ClutterTimeline that drives the state transition
+ * Return value: (transfer none): the #ClutterTimeline that drives the
+ *   state transition. The returned timeline is owned by the #ClutterState
+ *   and it should not be unreferenced
  *
  * Since: 1.4
  */
@@ -581,7 +594,9 @@ clutter_state_set_state (ClutterState *state,
  *
  * Change to the specified target state immediately with no animation.
  *
- * Return value: the #ClutterTimeline that drives the state transition
+ * Return value: (transfer none): the #ClutterTimeline that drives the
+ *   state transition. The returned timeline is owned by the #ClutterState
+ *   and it should not be unreferenced
  *
  * Since: 1.4
  */
@@ -881,8 +896,8 @@ clutter_state_fetch_state (ClutterState *state,
  * Sets one specific end key for a state_name, object, property_name
  * combination.
  *
- * Return value: the #ClutterState instance, allowing chaining of
- *   multiple calls
+ * Return value: (transfer none): the #ClutterState instance, allowing
+ *   chaining of multiple calls
  *
  * Since: 1.4
  */
@@ -970,7 +985,7 @@ clutter_state_get_states (ClutterState *state)
  * Returns a list of pointers to opaque structures with accessor functions
  * that describe the keys added to an animator.
  *
- * Return value: (transfer container) (element-type ClutterStateKey): a
+ * Return value: (transfer container) (element-type Clutter.StateKey): a
  *   newly allocated #GList of #ClutterStateKey<!-- -->s. The contents of
  *   the returned list are owned by the #ClutterState and should not be
  *   modified or freed. Use g_list_free() to free the resources allocated
@@ -1274,8 +1289,8 @@ clutter_state_get_animator (ClutterState *state,
  * the two named states.
  *
  * The @animator allows specifying a transition between the state that is
- * more elaborate than the basic transitions other allowed by the simple
- * tweening of properties defined in the #ClutterState keys.
+ * more elaborate than the basic transitions allowed by the tweening of
+ * properties defined in the #ClutterState keys.
  *
  * If @animator is %NULL it will unset an existing animator.
  *
@@ -1298,7 +1313,7 @@ clutter_state_set_animator (ClutterState    *state,
   source_state_name = g_intern_string (source_state_name);
   target_state_name = g_intern_string (target_state_name);
 
-  target_state = clutter_state_fetch_state (state, target_state_name, FALSE);
+  target_state = clutter_state_fetch_state (state, target_state_name, TRUE);
   if (target_state == NULL)
     return;
   
@@ -1323,8 +1338,11 @@ clutter_state_set_animator (ClutterState    *state,
 
   if (animator != NULL)
     {
-      StateAnimator state_animator =
-                              {source_state_name, g_object_ref (animator)};
+      StateAnimator state_animator = {
+        source_state_name,
+        g_object_ref (animator)
+      };
+
       g_array_append_val (target_state->animators, state_animator);
     }
 }
@@ -1747,21 +1765,12 @@ parse_state_transition (JsonArray *array,
 
   if (!json_object_has_member (object, "source") ||
       !json_object_has_member (object, "target") ||
-      !json_object_has_member (object, "keys"))
+      !(json_object_has_member (object, "keys") ||
+        json_object_has_member (object, "animator")))
     {
       g_warning ("The transition description at index %d is missing one "
-                 "of the mandatory members: source, target and keys",
-                 index_);
-      return;
-    }
-
-  keys = json_object_get_array_member (object, "keys");
-  if (keys == NULL)
-    {
-      g_warning ("The transition description at index %d has an invalid "
-                 "key member of type '%s' when an array was expected.",
-                 index_,
-                 json_node_type_name (json_object_get_member (object, "keys")));
+                 "of the mandatory members: source, target and keys or "
+                 "animator", index_);
       return;
     }
 
@@ -1798,6 +1807,19 @@ parse_state_transition (JsonArray *array,
                                   CLUTTER_ANIMATOR (animator));
     }
 
+  if (!json_object_has_member (object, "keys"))
+    return;
+
+  keys = json_object_get_array_member (object, "keys");
+  if (keys == NULL && !json_object_has_member (object, "animator"))
+    {
+      g_warning ("The transition description at index %d has an invalid "
+                 "key member of type '%s' when an array was expected.",
+                 index_,
+                 json_node_type_name (json_object_get_member (object, "keys")));
+      return;
+    }
+
   if (G_IS_VALUE (clos->value))
     valid_keys = g_slist_reverse (g_value_get_pointer (clos->value));
   else
