@@ -413,39 +413,53 @@ set_viewport_with_buffer_under_fbo_source (ClutterActor *fbo_source,
                                            int viewport_width,
                                            int viewport_height)
 {
-  ClutterVertex verts[4];
-  float x_min = G_MAXFLOAT, y_min = G_MAXFLOAT;
-  int x_offset, y_offset;
-  int i;
+  ClutterActorBox box = { 0, };
+  float x_offset, y_offset;
 
-  /* Get the actors allocation transformed into screen coordinates.
-   *
-   * XXX: Note: this may not be a bounding box for the actor, since an
-   * actor with depth may escape the box due to its perspective
-   * projection. */
-  clutter_actor_get_abs_allocation_vertices (fbo_source, verts);
-
-  for (i = 0; i < G_N_ELEMENTS (verts); ++i)
+  if (clutter_actor_get_paint_box (fbo_source, &box))
+    clutter_actor_box_get_origin (&box, &x_offset, &y_offset);
+  else
     {
-      if (verts[i].x < x_min)
-	x_min = verts[i].x;
-      if (verts[i].y < y_min)
-	y_min = verts[i].y;
-    }
+      /* As a fallback when the paint box can't be determined we use
+       * the transformed allocation to come up with an offset instead.
+       *
+       * FIXME: when we don't have a paint box we should instead be
+       * falling back to a stage sized fbo with an offset of (0,0)
+       */
 
-  /* XXX: It's not good enough to round by simply truncating the fraction here
-   * via a cast, as it results in offscreen rendering being offset by 1 pixel
-   * in many cases... */
+      ClutterVertex verts[4];
+      float x_min = G_MAXFLOAT, y_min = G_MAXFLOAT;
+      int i;
+
+      /* Get the actors allocation transformed into screen coordinates.
+       *
+       * XXX: Note: this may not be a bounding box for the actor, since an
+       * actor with depth may escape the box due to its perspective
+       * projection. */
+      clutter_actor_get_abs_allocation_vertices (fbo_source, verts);
+
+      for (i = 0; i < G_N_ELEMENTS (verts); ++i)
+        {
+          if (verts[i].x < x_min)
+           x_min = verts[i].x;
+          if (verts[i].y < y_min)
+           y_min = verts[i].y;
+        }
+
+      /* XXX: It's not good enough to round by simply truncating the fraction here
+       * via a cast, as it results in offscreen rendering being offset by 1 pixel
+       * in many cases... */
 #define ROUND(x) ((x) >= 0 ? (long)((x) + 0.5) : (long)((x) - 0.5))
 
-  x_offset = ROUND (-x_min);
-  y_offset = ROUND (-y_min);
+      x_offset = ROUND (x_min);
+      y_offset = ROUND (y_min);
 
 #undef ROUND
+    }
 
   /* translate the viewport so that the source actor lands on the
    * sub-region backed by the offscreen framebuffer... */
-  cogl_set_viewport (x_offset, y_offset, viewport_width, viewport_height);
+  cogl_set_viewport (-x_offset, -y_offset, viewport_width, viewport_height);
 }
 
 static void
@@ -456,7 +470,7 @@ update_fbo (ClutterActor *self)
   ClutterMainContext    *context;
   ClutterShader         *shader = NULL;
   ClutterActor          *stage = NULL;
-  ClutterPerspective     perspective;
+  CoglMatrix             projection;
   CoglColor              transparent_col;
 
   context = _clutter_context_get_default ();
@@ -490,24 +504,21 @@ update_fbo (ClutterActor *self)
        * under the actor.
        */
 
-      clutter_stage_get_perspective (CLUTTER_STAGE (stage), &perspective);
+      _clutter_stage_get_projection_matrix (CLUTTER_STAGE (stage), &projection);
+
+      /* Set the projection matrix modelview matrix as it is for the
+       * stage... */
+      cogl_set_projection_matrix (&projection);
+
       clutter_actor_get_size (stage, &stage_width, &stage_height);
 
-      /* Set the projection matrix modelview matrix and viewport size as
-       * they are for the stage... */
-      _cogl_setup_viewport (stage_width, stage_height,
-                            perspective.fovy,
-                            perspective.aspect,
-                            perspective.z_near,
-                            perspective.z_far);
-
-      /* Negatively offset the viewport so that the offscreen framebuffer is
-       * position underneath the fbo_source actor... */
+      /* Set a negatively offset the viewport so that the offscreen
+       * framebuffer is position underneath the fbo_source actor... */
       set_viewport_with_buffer_under_fbo_source (priv->fbo_source,
                                                  stage_width,
                                                  stage_height);
 
-      /* Reapply the source's parent transformations */
+      /* Apply the source's parent transformations to the modelview */
       if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
         {
           CoglMatrix modelview;
@@ -674,6 +685,15 @@ clutter_texture_paint (ClutterActor *self)
   gen_texcoords_and_draw_cogl_rectangle (self);
 }
 
+static gboolean
+clutter_texture_get_paint_volume (ClutterActor       *self,
+                                  ClutterPaintVolume *volume)
+{
+  return _clutter_actor_set_default_paint_volume (self,
+                                                  CLUTTER_TYPE_TEXTURE,
+                                                  volume);
+}
+
 static void
 clutter_texture_async_data_free (ClutterTextureAsyncData *data)
 {
@@ -945,10 +965,11 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   g_type_class_add_private (klass, sizeof (ClutterTexturePrivate));
 
-  actor_class->paint          = clutter_texture_paint;
-  actor_class->pick           = clutter_texture_pick;
-  actor_class->realize        = clutter_texture_realize;
-  actor_class->unrealize      = clutter_texture_unrealize;
+  actor_class->paint            = clutter_texture_paint;
+  actor_class->pick             = clutter_texture_pick;
+  actor_class->get_paint_volume = clutter_texture_get_paint_volume;
+  actor_class->realize          = clutter_texture_realize;
+  actor_class->unrealize        = clutter_texture_unrealize;
 
   actor_class->get_preferred_width  = clutter_texture_get_preferred_width;
   actor_class->get_preferred_height = clutter_texture_get_preferred_height;
@@ -1271,6 +1292,7 @@ clutter_texture_init (ClutterTexture *self)
       cogl_handle_unref (dummy_tex);
     }
 
+  g_assert (texture_template_material != NULL);
   priv->material = cogl_material_copy (texture_template_material);
 }
 
@@ -1356,13 +1378,16 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
 CoglHandle
 clutter_texture_get_cogl_texture (ClutterTexture *texture)
 {
+  ClutterTexturePrivate *priv;
   CoglMaterialLayerType layer_type;
   const GList *layers;
   int n_layers;
 
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), COGL_INVALID_HANDLE);
 
-  layers = cogl_material_get_layers (texture->priv->material);
+  priv = texture->priv;
+
+  layers = cogl_material_get_layers (priv->material);
   if (layers == NULL)
     return COGL_INVALID_HANDLE;
 
@@ -1421,6 +1446,10 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
   texture_free_gl_resources (texture);
 
   /* Use the new texture */
+  if (priv->material == NULL)
+    priv->material = cogl_material_copy (texture_template_material);
+
+  g_assert (priv->material != NULL);
   cogl_material_set_layer (priv->material, 0, cogl_tex);
 
   /* The material now holds a reference to the texture so we can
@@ -2330,8 +2359,27 @@ on_fbo_source_size_change (GObject          *object,
 {
   ClutterTexturePrivate *priv = texture->priv;
   gfloat w, h;
+  ClutterActorBox box;
+  gboolean status;
 
-  clutter_actor_get_transformed_size (priv->fbo_source, &w, &h);
+  status = clutter_actor_get_paint_box (priv->fbo_source, &box);
+  if (status)
+    clutter_actor_box_get_size (&box, &w, &h);
+
+  /* In the end we will size the framebuffer according to the paint
+   * box, but for code that does:
+   *   tex = clutter_texture_new_from_actor (src);
+   *   clutter_actor_get_size (tex, &width, &height);
+   * it seems more helpfull to return the src actor size if it has a
+   * degenerate paint box. The most likely reason it will have a
+   * degenerate paint box is simply that the src currently has no
+   * parent. */
+  if (status == FALSE || w == 0 || h == 0)
+    clutter_actor_get_size (priv->fbo_source, &w, &h);
+
+  /* We can't create a texture with a width or height of 0... */
+  w = MAX (1, w);
+  h = MAX (1, h);
 
   if (w != priv->image_width || h != priv->image_height)
     {
@@ -2388,6 +2436,21 @@ on_fbo_parent_change (ClutterActor        *actor,
       }
 }
 
+static void
+fbo_source_queue_redraw_cb (ClutterActor *source,
+			    ClutterActor *origin,
+			    ClutterTexture *texture)
+{
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+}
+
+static void
+fbo_source_queue_relayout_cb (ClutterActor *source,
+			      ClutterTexture *texture)
+{
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
+}
+
 /**
  * clutter_texture_new_from_actor:
  * @actor: A source #ClutterActor
@@ -2471,6 +2534,8 @@ clutter_texture_new_from_actor (ClutterActor *actor)
   ClutterTexture        *texture;
   ClutterTexturePrivate *priv;
   gfloat w, h;
+  ClutterActorBox box;
+  gboolean status;
 
   g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), NULL);
 
@@ -2485,10 +2550,25 @@ clutter_texture_new_from_actor (ClutterActor *actor)
 	return NULL;
     }
 
-  clutter_actor_get_transformed_size (actor, &w, &h);
+  status = clutter_actor_get_paint_box (actor, &box);
+  if (status)
+    clutter_actor_box_get_size (&box, &w, &h);
 
-  if (w == 0 || h == 0)
-    return NULL;
+  /* In the end we will size the framebuffer according to the paint
+   * box, but for code that does:
+   *   tex = clutter_texture_new_from_actor (src);
+   *   clutter_actor_get_size (tex, &width, &height);
+   * it seems more helpfull to return the src actor size if it has a
+   * degenerate paint box. The most likely reason it will have a
+   * degenerate paint box is simply that the src currently has no
+   * parent. */
+  if (status == FALSE || w == 0 || h == 0)
+    clutter_actor_get_size (actor, &w, &h);
+
+  /* We can't create a 0x0 fbo so always bump the size up to at least
+   * 1 */
+  w = MAX (1, w);
+  h = MAX (1, h);
 
   /* Hopefully now were good.. */
   texture = g_object_new (CLUTTER_TYPE_TEXTURE,
@@ -2534,6 +2614,11 @@ clutter_texture_new_from_actor (ClutterActor *actor)
                     G_CALLBACK(on_fbo_source_size_change),
                     texture);
 
+  g_signal_connect (actor, "queue-relayout",
+                    G_CALLBACK (fbo_source_queue_relayout_cb), texture);
+  g_signal_connect (actor, "queue-redraw",
+                    G_CALLBACK (fbo_source_queue_redraw_cb), texture);
+
   /* And a warning if the source becomes a child of the texture */
   g_signal_connect (actor,
                     "parent-set",
@@ -2577,6 +2662,16 @@ texture_fbo_free_resources (ClutterTexture *texture)
                              G_CALLBACK(on_fbo_source_size_change),
                              texture);
 
+      g_signal_handlers_disconnect_by_func
+                            (priv->fbo_source,
+                             G_CALLBACK(fbo_source_queue_relayout_cb),
+                             texture);
+
+      g_signal_handlers_disconnect_by_func
+                            (priv->fbo_source,
+                             G_CALLBACK(fbo_source_queue_redraw_cb),
+                             texture);
+
       g_object_unref (priv->fbo_source);
 
       priv->fbo_source = NULL;
