@@ -26,11 +26,13 @@
 #endif
 
 #include "cogl.h"
+#include "cogl-debug.h"
 #include "cogl-internal.h"
 #include "cogl-context.h"
 #include "cogl-journal-private.h"
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
+#include "cogl-material-opengl-private.h"
 #include "cogl-vertex-buffer-private.h"
 #include "cogl-framebuffer-private.h"
 
@@ -57,6 +59,7 @@ typedef struct _TextureSlicedQuadState
   float quad_len_y;
   gboolean flipped_x;
   gboolean flipped_y;
+  CoglMaterialWrapModeOverrides *wrap_mode_overrides;
 } TextureSlicedQuadState;
 
 typedef struct _TextureSlicedPolygonState
@@ -66,7 +69,6 @@ typedef struct _TextureSlicedPolygonState
   int stride;
 } TextureSlicedPolygonState;
 
-
 static void
 log_quad_sub_textures_cb (CoglHandle texture_handle,
                           GLuint gl_handle,
@@ -117,6 +119,7 @@ log_quad_sub_textures_cb (CoglHandle texture_handle,
                           1, /* one layer */
                           0, /* don't need to use fallbacks */
                           gl_handle, /* replace the layer0 texture */
+                          state->wrap_mode_overrides, /* use GL_CLAMP_TO_EDGE */
                           subtexture_coords,
                           4);
 }
@@ -138,6 +141,8 @@ log_quad_sub_textures_cb (CoglHandle texture_handle,
 static void
 _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
                                         CoglHandle   material,
+                                        gboolean     clamp_s,
+                                        gboolean     clamp_t,
                                         const float *position,
                                         float        tx_1,
                                         float        ty_1,
@@ -145,19 +150,135 @@ _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
                                         float        ty_2)
 {
   TextureSlicedQuadState state;
+  CoglMaterialWrapModeOverrides wrap_mode_overrides;
   gboolean tex_virtual_flipped_x;
   gboolean tex_virtual_flipped_y;
   gboolean quad_flipped_x;
   gboolean quad_flipped_y;
+  CoglHandle first_layer;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  COGL_NOTE (DRAW, "Drawing Tex Quad (Multi-Prim Mode)");
+  /* If the wrap mode is clamp to edge then we'll recursively draw the
+     stretched part and replace the coordinates */
+  if (clamp_s && tx_1 != tx_2)
+    {
+      float *replacement_position = g_newa (float, 4);
+      float old_tx_1 = tx_1, old_tx_2 = tx_2;
+
+      memcpy (replacement_position, position, sizeof (float) * 4);
+
+      tx_1 = CLAMP (tx_1, 0.0f, 1.0f);
+      tx_2 = CLAMP (tx_2, 0.0f, 1.0f);
+
+      if (old_tx_1 != tx_1)
+        {
+          /* Draw the left part of the quad as a stretched copy of tx_1 */
+          float tmp_position[] =
+            { position[0], position[1],
+              (position[0] +
+               (position[2] - position[0]) *
+               (tx_1 - old_tx_1) / (old_tx_2 - old_tx_1)),
+              position[3] };
+          _cogl_texture_quad_multiple_primitives (tex_handle, material,
+                                                  FALSE, clamp_t,
+                                                  tmp_position,
+                                                  tx_1, ty_1, tx_1, ty_2);
+          replacement_position[0] = tmp_position[2];
+        }
+
+      if (old_tx_2 != tx_2)
+        {
+          /* Draw the right part of the quad as a stretched copy of tx_2 */
+          float tmp_position[] =
+            { (position[0] +
+               (position[2] - position[0]) *
+               (tx_2 - old_tx_1) / (old_tx_2 - old_tx_1)),
+              position[1], position[2], position[3] };
+          _cogl_texture_quad_multiple_primitives (tex_handle, material,
+                                                  FALSE, clamp_t,
+                                                  tmp_position,
+                                                  tx_2, ty_1, tx_2, ty_2);
+          replacement_position[2] = tmp_position[0];
+        }
+
+      /* If there's no main part left then we don't need to continue */
+      if (tx_1 == tx_2)
+        return;
+
+      position = replacement_position;
+    }
+
+  if (clamp_t && ty_1 != ty_2)
+    {
+      float *replacement_position = g_newa (float, 4);
+      float old_ty_1 = ty_1, old_ty_2 = ty_2;
+
+      memcpy (replacement_position, position, sizeof (float) * 4);
+
+      ty_1 = CLAMP (ty_1, 0.0f, 1.0f);
+      ty_2 = CLAMP (ty_2, 0.0f, 1.0f);
+
+      if (old_ty_1 != ty_1)
+        {
+          /* Draw the top part of the quad as a stretched copy of ty_1 */
+          float tmp_position[] =
+            { position[0], position[1], position[2],
+              (position[1] +
+               (position[3] - position[1]) *
+               (ty_1 - old_ty_1) / (old_ty_2 - old_ty_1)) };
+          _cogl_texture_quad_multiple_primitives (tex_handle, material,
+                                                  clamp_s, FALSE,
+                                                  tmp_position,
+                                                  tx_1, ty_1, tx_2, ty_1);
+          replacement_position[1] = tmp_position[3];
+        }
+
+      if (old_ty_2 != ty_2)
+        {
+          /* Draw the bottom part of the quad as a stretched copy of ty_2 */
+          float tmp_position[] =
+            { position[0],
+              (position[1] +
+               (position[3] - position[1]) *
+               (ty_2 - old_ty_1) / (old_ty_2 - old_ty_1)),
+              position[2], position[3] };
+          _cogl_texture_quad_multiple_primitives (tex_handle, material,
+                                                  clamp_s, FALSE,
+                                                  tmp_position,
+                                                  tx_1, ty_2, tx_2, ty_2);
+          replacement_position[3] = tmp_position[1];
+        }
+
+      /* If there's no main part left then we don't need to continue */
+      if (ty_1 == ty_2)
+        return;
+
+      position = replacement_position;
+    }
+
+  state.wrap_mode_overrides = NULL;
+  memset (&wrap_mode_overrides, 0, sizeof (wrap_mode_overrides));
 
   /* We can't use hardware repeat so we need to set clamp to edge
-     otherwise it might pull in edge pixels from the other side */
-  /* FIXME: wrap modes should be part of the material! */
-  _cogl_texture_set_wrap_mode_parameter (tex_handle, GL_CLAMP_TO_EDGE);
+     otherwise it might pull in edge pixels from the other side. By
+     default WRAP_MODE_AUTOMATIC becomes CLAMP_TO_EDGE so we only need
+     to override if the wrap mode is repeat */
+  first_layer = cogl_material_get_layers (material)->data;
+  if (cogl_material_layer_get_wrap_mode_s (first_layer) ==
+      COGL_MATERIAL_WRAP_MODE_REPEAT)
+    {
+      state.wrap_mode_overrides = &wrap_mode_overrides;
+      wrap_mode_overrides.values[0].s =
+        COGL_MATERIAL_WRAP_MODE_OVERRIDE_CLAMP_TO_EDGE;
+    }
+  if (cogl_material_layer_get_wrap_mode_t (first_layer) ==
+      COGL_MATERIAL_WRAP_MODE_REPEAT)
+    {
+      state.wrap_mode_overrides = &wrap_mode_overrides;
+      wrap_mode_overrides.values[0].t =
+        COGL_MATERIAL_WRAP_MODE_OVERRIDE_CLAMP_TO_EDGE;
+    }
 
   state.material = material;
 
@@ -236,9 +357,15 @@ _cogl_multitexture_quad_single_primitive (const float *position,
   const GList *layers;
   GList       *tmp;
   int          i;
+  CoglMaterialWrapModeOverrides wrap_mode_overrides;
+  /* This will be set to point to wrap_mode_overrides when an override
+     is needed */
+  CoglMaterialWrapModeOverrides *wrap_mode_overrides_p = NULL;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
+  memset (&wrap_mode_overrides, 0, sizeof (wrap_mode_overrides));
+
   /*
    * Validate the texture coordinates for this rectangle.
    */
@@ -251,7 +378,6 @@ _cogl_multitexture_quad_single_primitive (const float *position,
       float              *out_tex_coords;
       float               default_tex_coords[4] = {0.0, 0.0, 1.0, 1.0};
       CoglTransformResult transform_result;
-      GLenum              wrap_mode;
 
       tex_handle = cogl_material_layer_get_texture (layer);
 
@@ -321,15 +447,28 @@ _cogl_multitexture_quad_single_primitive (const float *position,
             }
         }
 
-      /* If we're not repeating then we want to clamp the coords
-         to the edge otherwise it can pull in edge pixels from the
-         wrong side when scaled */
+      /* By default WRAP_MODE_AUTOMATIC becomes to CLAMP_TO_EDGE. If
+         the texture coordinates need repeating then we'll override
+         this to GL_REPEAT. Otherwise we'll leave it at CLAMP_TO_EDGE
+         so that it won't blend in pixels from the opposite side when
+         the full texture is drawn with GL_LINEAR filter mode */
       if (transform_result == COGL_TRANSFORM_HARDWARE_REPEAT)
-        wrap_mode = GL_REPEAT;
-      else
-        wrap_mode = GL_CLAMP_TO_EDGE;
-
-      _cogl_texture_set_wrap_mode_parameter (tex_handle, wrap_mode);
+        {
+          if (cogl_material_layer_get_wrap_mode_s (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              wrap_mode_overrides.values[i].s
+                = COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              wrap_mode_overrides_p = &wrap_mode_overrides;
+            }
+          if (cogl_material_layer_get_wrap_mode_t (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              wrap_mode_overrides.values[i].t
+                = COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              wrap_mode_overrides_p = &wrap_mode_overrides;
+            }
+        }
     }
 
   _cogl_journal_log_quad (position,
@@ -337,6 +476,7 @@ _cogl_multitexture_quad_single_primitive (const float *position,
                           n_layers,
                           fallback_layers,
                           0, /* don't replace the layer0 texture */
+                          wrap_mode_overrides_p,
                           final_tex_coords,
                           n_layers * 4);
 
@@ -378,17 +518,52 @@ _cogl_rectangles_with_multitexture_coords (
     {
       CoglHandle     layer = tmp->data;
       CoglHandle     tex_handle;
-      unsigned long  flags;
 
       if (cogl_material_layer_get_type (layer)
 	  != COGL_MATERIAL_LAYER_TYPE_TEXTURE)
 	continue;
 
       /* We need to ensure the mipmaps are ready before deciding
-         anything else about the texture because it could become
-         something completely different if it needs to be migrated out
-         of the atlas */
-      _cogl_material_layer_ensure_mipmaps (layer);
+       * anything else about the texture because the texture storage
+       * could completely change if it needs to be migrated out of the
+       * atlas and will affect how we validate the layer.
+       *
+       * FIXME: this needs to be generalized. There could be any
+       * number of things that might require a shuffling of the
+       * underlying texture storage. We could add two mechanisms to
+       * generalize this a bit...
+       *
+       * 1) add a _cogl_material_layer_update_storage() function that
+       * would for instance consider if mipmapping is necessary and
+       * potentially migrate the texture from an atlas.
+       *
+       * 2) allow setting of transient primitive-flags on a material
+       * that may affect the outcome of _update_storage(). One flag
+       * could indicate that we expect to sample beyond the bounds of
+       * the texture border.
+       *
+       *   flags = COGL_MATERIAL_PRIMITIVE_FLAG_VALID_BORDERS;
+       *   _cogl_material_layer_assert_primitive_flags (layer, flags)
+       *   _cogl_material_layer_update_storage (layer)
+       *   enqueue primitive in journal
+       *
+       *   when the primitive is dequeued and drawn we should:
+       *   _cogl_material_flush_gl_state (material)
+       *   draw primitive
+       *   _cogl_material_unassert_primitive_flags (layer, flags);
+       *
+       * _cogl_material_layer_update_storage should take into
+       * consideration all the asserted primitive requirements.  (E.g.
+       * there could be multiple primitives in the journal - or in a
+       * renderlist in the future - that need mipmaps or that need
+       * valid contents beyond their borders (for cogl_polygon)
+       * meaning they can't work with textures in an atas, so
+       * _cogl_material_layer_update_storage would pass on these
+       * requirements to the texture atlas backend which would make
+       * sure the referenced texture is migrated out of the atlas and
+       * mipmaps are generated.)
+       */
+      _cogl_material_layer_pre_paint (layer);
 
       tex_handle = cogl_material_layer_get_texture (layer);
 
@@ -445,8 +620,7 @@ _cogl_rectangles_with_multitexture_coords (
        * waste or if using GL_TEXTURE_RECTANGLE_ARB) then we don't support
        * multi texturing since we don't know if the result will end up trying
        * to texture from the waste area. */
-      flags = _cogl_material_layer_get_flags (layer);
-      if (flags & COGL_MATERIAL_LAYER_FLAG_HAS_USER_MATRIX
+      if (_cogl_material_layer_has_user_matrix (layer)
           && !_cogl_texture_can_hardware_repeat (tex_handle))
         {
           static gboolean warning_seen = FALSE;
@@ -474,6 +648,7 @@ _cogl_rectangles_with_multitexture_coords (
       CoglHandle   first_layer, tex_handle;
       const float  default_tex_coords[4] = {0.0, 0.0, 1.0, 1.0};
       const float *tex_coords;
+      gboolean     clamp_s, clamp_t;
 
       if (!all_use_sliced_quad_fallback)
         {
@@ -495,6 +670,7 @@ _cogl_rectangles_with_multitexture_coords (
       /* If multitexturing failed or we are drawing with a sliced texture
        * then we only support a single layer so we pluck out the texture
        * from the first material layer... */
+      layers = cogl_material_get_layers (material);
       first_layer = layers->data;
       tex_handle = cogl_material_layer_get_texture (first_layer);
 
@@ -503,8 +679,16 @@ _cogl_rectangles_with_multitexture_coords (
       else
         tex_coords = default_tex_coords;
 
+      clamp_s = (cogl_material_layer_get_wrap_mode_s (first_layer) ==
+                 COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE);
+      clamp_t = (cogl_material_layer_get_wrap_mode_t (first_layer) ==
+                 COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE);
+
+      COGL_NOTE (DRAW, "Drawing Tex Quad (Multi-Prim Mode)");
+
       _cogl_texture_quad_multiple_primitives (tex_handle,
                                               material,
+                                              clamp_s, clamp_t,
                                               rects[i].position,
                                               tex_coords[0],
                                               tex_coords[1],
@@ -655,6 +839,7 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
   float virtual_origin_y;
   float v_to_s_scale_x;
   float v_to_s_scale_y;
+  CoglHandle source;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -683,16 +868,52 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
       v += state->stride;
     }
 
+  if (G_UNLIKELY (ctx->legacy_state_set))
+    {
+      source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_legacy_state (source);
+    }
+  else
+    source = ctx->source_material;
+
   options.flags =
-    COGL_MATERIAL_FLUSH_DISABLE_MASK |
-    COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE;
-  /* disable all except the first layer */
-  options.disable_layers = (guint32)~1;
+    COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE |
+    COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+
   options.layer0_override_texture = gl_handle;
 
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
+  /* Override the wrapping mode on all of the slices to use a
+     transparent border so that we can draw the full polygon for
+     each slice. Coordinates outside the texture will be transparent
+     so only the part of the polygon that intersects the slice will
+     be visible. This is a fairly hacky fallback and it relies on
+     the blending function working correctly */
+
+  memset (&options.wrap_mode_overrides, 0,
+          sizeof (options.wrap_mode_overrides));
+  options.wrap_mode_overrides.values[0].s =
+    COGL_MATERIAL_WRAP_MODE_OVERRIDE_CLAMP_TO_BORDER;
+  options.wrap_mode_overrides.values[0].t =
+    COGL_MATERIAL_WRAP_MODE_OVERRIDE_CLAMP_TO_BORDER;
+
+  if (cogl_material_get_n_layers (source) != 1)
+    {
+      /* disable all except the first layer */
+      options.disable_layers = (guint32)~1;
+      options.flags |= COGL_MATERIAL_FLUSH_DISABLE_MASK;
+    }
+
+  /* If we haven't already created a derived material... */
+  if (source == ctx->source_material)
+    source = cogl_material_copy (ctx->source_material);
+  _cogl_material_apply_overrides (source, &options);
+
+  _cogl_material_flush_gl_state (source, FALSE);
 
   GE (glDrawArrays (GL_TRIANGLE_FAN, 0, state->n_vertices));
+
+  if (G_UNLIKELY (source != ctx->source_material))
+    cogl_handle_unref (source);
 }
 
 /* handles 2d-sliced textures with > 1 slice */
@@ -755,7 +976,9 @@ _cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
                                              unsigned int n_layers,
                                              unsigned int stride,
                                              gboolean use_color,
-                                             guint32 fallback_layers)
+                                             guint32 fallback_layers,
+                                             CoglMaterialWrapModeOverrides *
+                                               wrap_mode_overrides)
 {
   CoglHandle           material;
   const GList         *layers;
@@ -763,6 +986,7 @@ _cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
   GList               *tmp;
   GLfloat             *v;
   CoglMaterialFlushOptions options;
+  CoglHandle           source;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -819,13 +1043,40 @@ _cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
         }
     }
 
-  options.flags = COGL_MATERIAL_FLUSH_FALLBACK_MASK;
-  if (use_color)
-    options.flags |= COGL_MATERIAL_FLUSH_SKIP_GL_COLOR;
-  options.fallback_layers = fallback_layers;
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
+  if (G_UNLIKELY (ctx->legacy_state_set))
+    {
+      source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_legacy_state (source);
+    }
+  else
+    source = ctx->source_material;
+
+  options.flags = 0;
+
+  if (G_UNLIKELY (fallback_layers))
+    {
+      options.flags |= COGL_MATERIAL_FLUSH_FALLBACK_MASK;
+      options.fallback_layers = fallback_layers;
+    }
+  if (wrap_mode_overrides)
+    {
+      options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+      options.wrap_mode_overrides = *wrap_mode_overrides;
+    }
+  if (options.flags)
+    {
+      /* If we haven't already created a derived material... */
+      if (source == ctx->source_material)
+        source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_overrides (source, &options);
+    }
+
+  _cogl_material_flush_gl_state (source, use_color);
 
   GE (glDrawArrays (GL_TRIANGLE_FAN, 0, n_vertices));
+
+  if (G_UNLIKELY (source != ctx->source_material))
+    cogl_handle_unref (source);
 }
 
 void
@@ -843,7 +1094,10 @@ cogl_polygon (const CoglTextureVertex *vertices,
   unsigned int         stride;
   gsize                stride_bytes;
   GLfloat             *v;
-  int                  prev_n_texcoord_arrays_enabled;
+  CoglMaterialWrapModeOverrides wrap_mode_overrides;
+  CoglMaterialWrapModeOverrides *wrap_mode_overrides_p = NULL;
+  CoglHandle           original_source_material = NULL;
+  gboolean             overrode_material = FALSE;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -858,6 +1112,8 @@ cogl_polygon (const CoglTextureVertex *vertices,
   layers = cogl_material_get_layers (ctx->source_material);
   n_layers = g_list_length ((GList *)layers);
 
+  memset (&wrap_mode_overrides, 0, sizeof (wrap_mode_overrides));
+
   for (tmp = layers, i = 0; tmp != NULL; tmp = tmp->next, i++)
     {
       CoglHandle layer = tmp->data;
@@ -869,10 +1125,22 @@ cogl_polygon (const CoglTextureVertex *vertices,
         continue;
 
       /* Give the texture a chance to know that we're rendering
-         non-quad shaped primitives. If the texture is in an atlas it
-         will be migrated */
+       * non-quad shaped primitives. If the texture is in an atlas it
+       * will be migrated
+       *
+       * FIXME: this needs to be generalized. There could be any
+       * number of things that might require a shuffling of the
+       * underlying texture storage.
+       */
       _cogl_texture_ensure_non_quad_rendering (tex_handle);
 
+      /* We need to ensure the mipmaps are ready before deciding
+       * anything else about the texture because the texture storate
+       * could completely change if it needs to be migrated out of the
+       * atlas and will affect how we validate the layer.
+       */
+      _cogl_material_layer_pre_paint (layer);
+
       if (i == 0 && cogl_texture_is_sliced (tex_handle))
         {
 #if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
@@ -914,16 +1182,6 @@ cogl_polygon (const CoglTextureVertex *vertices,
               return;
             }
 
-#ifdef HAVE_COGL_GL
-          {
-            /* Temporarily change the wrapping mode on all of the slices to use
-             * a transparent border
-             * XXX: it's doesn't look like we save/restore this, like
-             * the comment implies? */
-            _cogl_texture_set_wrap_mode_parameter (tex_handle,
-                                                   GL_CLAMP_TO_BORDER);
-          }
-#endif
           break;
         }
 
@@ -940,6 +1198,24 @@ cogl_polygon (const CoglTextureVertex *vertices,
           fallback_layers |= (1 << i);
           continue;
         }
+
+      /* By default COGL_MATERIAL_WRAP_MODE_AUTOMATIC becomes
+         GL_CLAMP_TO_EDGE but we want the polygon API to use GL_REPEAT
+         to maintain compatibility with previous releases */
+      if (cogl_material_layer_get_wrap_mode_s (layer) ==
+          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+        {
+          wrap_mode_overrides.values[i].s =
+            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+          wrap_mode_overrides_p = &wrap_mode_overrides;
+        }
+      if (cogl_material_layer_get_wrap_mode_t (layer) ==
+          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+        {
+          wrap_mode_overrides.values[i].t =
+            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+          wrap_mode_overrides_p = &wrap_mode_overrides;
+        }
     }
 
   /* Our data is arranged like:
@@ -955,21 +1231,34 @@ cogl_polygon (const CoglTextureVertex *vertices,
 
   /* Prepare GL state */
   enable_flags = COGL_ENABLE_VERTEX_ARRAY;
-  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
   if (use_color)
     {
-      enable_flags |= COGL_ENABLE_COLOR_ARRAY | COGL_ENABLE_BLEND;
+      CoglHandle override;
+      enable_flags |= COGL_ENABLE_COLOR_ARRAY;
       GE( glColorPointer (4, GL_UNSIGNED_BYTE,
                           stride_bytes,
                           /* NB: [X,Y,Z,TX,TY...,R,G,B,A,...] */
                           v + 3 + 2 * n_layers) );
+
+      if (!_cogl_material_get_real_blend_enabled (ctx->source_material))
+        {
+          CoglMaterialBlendEnable blend_enabled =
+            COGL_MATERIAL_BLEND_ENABLE_ENABLED;
+          original_source_material = ctx->source_material;
+          override = cogl_material_copy (original_source_material);
+          _cogl_material_set_blend_enabled (override, blend_enabled);
+
+          /* XXX: cogl_push_source () */
+          overrode_material = TRUE;
+          ctx->source_material = override;
+        }
     }
 
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   GE (glVertexPointer (3, GL_FLOAT, stride_bytes, v));
@@ -984,14 +1273,9 @@ cogl_polygon (const CoglTextureVertex *vertices,
                              v + 3 + 2 * i));
     }
 
-  prev_n_texcoord_arrays_enabled = ctx->n_texcoord_arrays_enabled;
-  ctx->n_texcoord_arrays_enabled = n_layers;
-
-  for (; i < prev_n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+  _cogl_bitmask_set_range (&ctx->temp_bitmask, n_layers, TRUE);
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 
   if (use_sliced_polygon_fallback)
     _cogl_texture_polygon_multiple_primitives (vertices,
@@ -1004,7 +1288,18 @@ cogl_polygon (const CoglTextureVertex *vertices,
                                                  n_layers,
                                                  stride,
                                                  use_color,
-                                                 fallback_layers);
+                                                 fallback_layers,
+                                                 wrap_mode_overrides_p);
+
+  /* XXX: cogl_pop_source () */
+  if (overrode_material)
+    {
+      cogl_handle_unref (ctx->source_material);
+      ctx->source_material = original_source_material;
+      /* XXX: when we have weak materials then any override material
+       * should get associated with the original material so we don't
+       * create lots of one-shot materials! */
+    }
 
   /* Reset the size of the logged vertex array because rendering
      rectangles expects it to start at 0 */
