@@ -62,7 +62,7 @@ static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 static ClutterStageWindowIface *clutter_stage_glx_parent_iface = NULL;
 
 G_DEFINE_TYPE_WITH_CODE (ClutterStageGLX,
-                         clutter_stage_glx,
+                         _clutter_stage_glx,
                          CLUTTER_TYPE_STAGE_X11,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_STAGE_WINDOW,
                                                 clutter_stage_window_iface_init));
@@ -276,11 +276,11 @@ clutter_stage_glx_get_pending_swaps (ClutterStageWindow *stage_window)
 static void
 clutter_stage_glx_dispose (GObject *gobject)
 {
-  G_OBJECT_CLASS (clutter_stage_glx_parent_class)->dispose (gobject);
+  G_OBJECT_CLASS (_clutter_stage_glx_parent_class)->dispose (gobject);
 }
 
 static void
-clutter_stage_glx_class_init (ClutterStageGLXClass *klass)
+_clutter_stage_glx_class_init (ClutterStageGLXClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
@@ -288,7 +288,7 @@ clutter_stage_glx_class_init (ClutterStageGLXClass *klass)
 }
 
 static void
-clutter_stage_glx_init (ClutterStageGLX *stage)
+_clutter_stage_glx_init (ClutterStageGLX *stage)
 {
 }
 
@@ -380,12 +380,11 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
   if (stage_clip == NULL)
     {
       stage_glx->bounding_redraw_clip.width = 0;
+      stage_glx->initialized_redraw_clip = TRUE;
       return;
     }
 
-  /* Do nothing on an empty clip to avoid confusing with out magic-flag
-   * degenerate clip
-   */
+  /* Ignore requests to add degenerate/empty clip rectangles */
   if (stage_clip->width == 0 || stage_clip->height == 0)
     return;
 
@@ -490,7 +489,7 @@ wait_for_vblank (ClutterBackendGLX *backend_glx)
 }
 
 void
-clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
+_clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                           ClutterStage *stage)
 {
   ClutterBackend    *backend;
@@ -499,6 +498,8 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
   ClutterStageX11   *stage_x11;
   GLXDrawable        drawable;
   unsigned int       video_sync_count;
+  gboolean           may_use_clipped_redraw;
+  gboolean           use_clipped_redraw;
   CLUTTER_STATIC_TIMER (painting_timer,
                         "Redrawing", /* parent */
                         "Painting actors",
@@ -523,21 +524,84 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
 
-  if (backend_glx->can_blit_sub_buffer &&
+  if (G_LIKELY (backend_glx->can_blit_sub_buffer) &&
       /* NB: a degenerate redraw clip width == full stage redraw */
-      (stage_glx->bounding_redraw_clip.width != 0) &&
+      stage_glx->bounding_redraw_clip.width != 0 &&
+      /* some drivers struggle to get going and produce some junk
+       * frames when starting up... */
+      G_LIKELY (stage_glx->frame_count > 3) &&
+      /* While resizing a window clipped redraws are disabled to avoid
+       * artefacts. See clutter-event-x11.c:event_translate for a
+       * detailed explanation */
+      G_LIKELY (stage_x11->clipped_redraws_cool_off == 0))
+    may_use_clipped_redraw = TRUE;
+  else
+    may_use_clipped_redraw = FALSE;
+
+  if (may_use_clipped_redraw &&
       G_LIKELY (!(clutter_paint_debug_flags &
                   CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS)))
+    use_clipped_redraw = TRUE;
+  else
+    use_clipped_redraw = FALSE;
+
+  if (use_clipped_redraw)
     {
       cogl_clip_push_window_rectangle (stage_glx->bounding_redraw_clip.x,
                                        stage_glx->bounding_redraw_clip.y,
                                        stage_glx->bounding_redraw_clip.width,
                                        stage_glx->bounding_redraw_clip.height);
-      clutter_actor_paint (CLUTTER_ACTOR (stage));
+      _clutter_stage_do_paint (stage, &stage_glx->bounding_redraw_clip);
       cogl_clip_pop ();
     }
   else
-    clutter_actor_paint (CLUTTER_ACTOR (stage));
+    _clutter_stage_do_paint (stage, NULL);
+
+  if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS &&
+      may_use_clipped_redraw)
+    {
+      ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
+      static CoglMaterial *outline = NULL;
+      CoglHandle vbo;
+      float x_1 = clip->x;
+      float x_2 = clip->x + clip->width;
+      float y_1 = clip->y;
+      float y_2 = clip->y + clip->height;
+      float quad[8] = {
+        x_1, y_1,
+        x_2, y_1,
+        x_2, y_2,
+        x_1, y_2
+      };
+      CoglMatrix modelview;
+
+      if (outline == NULL)
+        {
+          outline = cogl_material_new ();
+          cogl_material_set_color4ub (outline, 0xff, 0x00, 0x00, 0xff);
+        }
+
+      vbo = cogl_vertex_buffer_new (4);
+      cogl_vertex_buffer_add (vbo,
+                              "gl_Vertex",
+                              2, /* n_components */
+                              COGL_ATTRIBUTE_TYPE_FLOAT,
+                              FALSE, /* normalized */
+                              0, /* stride */
+                              quad);
+      cogl_vertex_buffer_submit (vbo);
+
+      cogl_push_matrix ();
+      cogl_matrix_init_identity (&modelview);
+      _clutter_actor_apply_modelview_transform (CLUTTER_ACTOR (stage),
+                                                &modelview);
+      cogl_set_modelview_matrix (&modelview);
+      cogl_set_source (outline);
+      cogl_vertex_buffer_draw (vbo, COGL_VERTICES_MODE_LINE_LOOP,
+                               0 , 4);
+      cogl_pop_matrix ();
+      cogl_object_unref (vbo);
+    }
 
   cogl_flush ();
   CLUTTER_TIMER_STOP (_clutter_uprof_context, painting_timer);
@@ -559,11 +623,7 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
     backend_glx->get_video_sync (&video_sync_count);
 
   /* push on the screen */
-  if (backend_glx->can_blit_sub_buffer &&
-      /* NB: a degenerate redraw clip width == full stage redraw */
-      (stage_glx->bounding_redraw_clip.width != 0) &&
-      G_LIKELY (!(clutter_paint_debug_flags &
-                  CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS)))
+  if (use_clipped_redraw)
     {
       ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
       ClutterGeometry copy_area;
@@ -578,43 +638,6 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                     stage_glx->bounding_redraw_clip.width,
                     stage_glx->bounding_redraw_clip.height);
 
-      if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS)
-        {
-          static CoglHandle outline = COGL_INVALID_HANDLE;
-          CoglHandle vbo;
-          float x_1 = clip->x;
-          float x_2 = clip->x + clip->width;
-          float y_1 = clip->y;
-          float y_2 = clip->y + clip->height;
-          float quad[8] = {
-            x_1, y_1,
-            x_2, y_1,
-            x_2, y_2,
-            x_1, y_2
-          };
-
-          if (outline == COGL_INVALID_HANDLE)
-            {
-              outline = cogl_material_new ();
-              cogl_material_set_color4ub (outline, 0xff, 0x00, 0x00, 0xff);
-            }
-
-          vbo = cogl_vertex_buffer_new (4);
-          cogl_vertex_buffer_add (vbo,
-                                  "gl_Vertex",
-                                  2, /* n_components */
-                                  COGL_ATTRIBUTE_TYPE_FLOAT,
-                                  FALSE, /* normalized */
-                                  0, /* stride */
-                                  quad);
-          cogl_vertex_buffer_submit (vbo);
-
-          cogl_set_source (outline);
-          cogl_vertex_buffer_draw (vbo, COGL_VERTICES_MODE_LINE_LOOP,
-                                   0 , 4);
-          cogl_flush ();
-        }
-
       /* XXX: It seems there will be a race here in that the stage
        * window may be resized before glXCopySubBufferMESA is handled
        * and so we may copy the wrong region. I can't really see how
@@ -719,5 +742,7 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   /* reset the redraw clipping for the next paint... */
   stage_glx->initialized_redraw_clip = FALSE;
+
+  stage_glx->frame_count++;
 }
 
