@@ -14,15 +14,19 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "clutter-backend-x11.h"
 #include "clutter-stage-x11.h"
 #include "clutter-x11.h"
@@ -66,6 +70,8 @@ send_wmspec_change_state (ClutterBackendX11 *backend_x11,
 {
   XClientMessageEvent xclient;
 
+  CLUTTER_NOTE (BACKEND, "%s NET_WM state", add ? "adding" : "removing");
+
   memset (&xclient, 0, sizeof (xclient));
 
   xclient.type         = ClientMessage;
@@ -86,6 +92,30 @@ send_wmspec_change_state (ClutterBackendX11 *backend_x11,
               (XEvent *)&xclient);
 }
 
+static void
+update_state (ClutterStageX11   *stage_x11,
+              ClutterBackendX11 *backend_x11,
+              Atom              *state,
+              gboolean           add)
+{
+  if (add)
+    {
+      /* FIXME: This wont work if we support more states */
+      XChangeProperty (backend_x11->xdpy,
+                       stage_x11->xwin,
+                       backend_x11->atom_NET_WM_STATE, XA_ATOM, 32,
+                       PropModeReplace,
+                       (unsigned char *) state, 1);
+    }
+  else
+    {
+       /* FIXME: This wont work if we support more states */
+       XDeleteProperty (backend_x11->xdpy,
+                        stage_x11->xwin,
+                        backend_x11->atom_NET_WM_STATE);
+    }
+}
+
 void
 clutter_stage_x11_fix_window_size (ClutterStageX11 *stage_x11,
                                    gint             new_width,
@@ -121,7 +151,7 @@ clutter_stage_x11_fix_window_size (ClutterStageX11 *stage_x11,
 
       /* If we are going fullscreen then we don't want any
          restrictions on the window size */
-      if (!stage_x11->fullscreen_on_map)
+      if (!stage_x11->fullscreening)
         {
           if (resize)
             {
@@ -169,17 +199,13 @@ clutter_stage_x11_get_geometry (ClutterStageWindow *stage_window,
   ClutterBackend *backend = clutter_get_default_backend ();
   ClutterBackendX11 *backend_x11;
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
-  gboolean is_fullscreen;
 
   g_return_if_fail (CLUTTER_IS_BACKEND_X11 (backend));
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
-  is_fullscreen = FALSE;
-  g_object_get (G_OBJECT (stage_x11->wrapper),
-                "fullscreen-set", &is_fullscreen,
-                NULL);
-
-  if (is_fullscreen || stage_x11->fullscreen_on_map)
+  /* If we're fullscreen, return the size of the display. */
+  if ((stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN) &&
+      stage_x11->fullscreening)
     {
       geometry->width = DisplayWidth (backend_x11->xdpy, backend_x11->xscreen_num);
       geometry->height = DisplayHeight (backend_x11->xdpy, backend_x11->xscreen_num);
@@ -199,9 +225,24 @@ clutter_stage_x11_resize (ClutterStageWindow *stage_window,
   ClutterBackend *backend = clutter_get_default_backend ();
   ClutterBackendX11 *backend_x11;
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
-  ClutterStage *stage = stage_x11->wrapper;
   gboolean resize;
 
+  if (stage_x11->is_foreign_xwin)
+    {
+      /* If this is a foreign window we won't get a ConfigureNotify,
+       * so we need to manually set the size and queue a relayout on the
+       * stage here (as is normally done in response to ConfigureNotify).
+       */
+      stage_x11->xwin_width = width;
+      stage_x11->xwin_height = height;
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (stage_x11->wrapper));
+      return;
+    }
+
+  /* If we're going fullscreen, don't mess with the size */
+  if (stage_x11->fullscreening)
+    return;
+
   resize = clutter_stage_get_user_resizable (stage_x11->wrapper);
 
   g_return_if_fail (CLUTTER_IS_BACKEND_X11 (backend));
@@ -219,7 +260,7 @@ clutter_stage_x11_resize (ClutterStageWindow *stage_window,
 
   CLUTTER_NOTE (BACKEND, "New size received: (%d, %d)", width, height);
 
-  if (stage_x11->xwin != None && !stage_x11->is_foreign_xwin)
+  if (stage_x11->xwin != None)
     {
       clutter_stage_x11_fix_window_size (stage_x11, width, height);
 
@@ -233,28 +274,16 @@ clutter_stage_x11_resize (ClutterStageWindow *stage_window,
                         height);
 
           CLUTTER_SET_PRIVATE_FLAGS (stage_x11->wrapper,
-                                     CLUTTER_STAGE_IN_RESIZE);
+                                     CLUTTER_IN_RESIZE);
 
+          /* XXX: in this case we can rely on a subsequent
+           * ConfigureNotify that will result in the stage
+           * being reallocated so we don't actively do anything
+           * to affect the stage allocation here. */
           XResizeWindow (backend_x11->xdpy,
                          stage_x11->xwin,
                          width,
                          height);
-
-          /* If the viewport hasn't previously been initialized then even
-           * though we can't guarantee that the server will honour our request
-           * we need to ensure a valid viewport is set before our first paint.
-           */
-          if (G_UNLIKELY (!stage_x11->viewport_initialized))
-            {
-              ClutterPerspective perspective;
-              clutter_stage_get_perspective (stage, &perspective);
-              _cogl_setup_viewport (width,
-                                    height,
-                                    perspective.fovy,
-                                    perspective.aspect,
-                                    perspective.z_near,
-                                    perspective.z_far);
-            }
         }
     }
 }
@@ -389,7 +418,10 @@ clutter_stage_x11_set_fullscreen (ClutterStageWindow *stage_window,
   if (stage == NULL)
     return;
 
-  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
+  if (!!(stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN) == is_fullscreen)
+    return;
+
+  CLUTTER_NOTE (BACKEND, "%ssetting fullscreen", is_fullscreen ? "" : "un");
 
   if (is_fullscreen)
     {
@@ -403,17 +435,8 @@ clutter_stage_x11_set_fullscreen (ClutterStageWindow *stage_window,
       width  = DisplayWidth (backend_x11->xdpy, backend_x11->xscreen_num);
       height = DisplayHeight (backend_x11->xdpy, backend_x11->xscreen_num);
 
-      /* we force the stage to the screen size here, in order to
-       * get the fullscreen stage size right after the call to
-       * clutter_stage_fullscreen(). XXX this might break in case
-       * the stage is not fullscreened, but if that does not happen
-       * we are massively screwed anyway
-       */
-      stage_x11->xwin_width = width;
-      stage_x11->xwin_height = height;
-
-      if (!STAGE_X11_IS_MAPPED (stage_x11))
-        stage_x11->fullscreen_on_map = TRUE;
+      /* Set the fullscreen hint so we can retain the old size of the window. */
+      stage_x11->fullscreening = TRUE;
 
       if (stage_x11->xwin != None)
         {
@@ -422,17 +445,18 @@ clutter_stage_x11_set_fullscreen (ClutterStageWindow *stage_window,
            * a resize when calling clutter_stage_fullscreen() before showing
            * the stage
            */
-          if (!CLUTTER_ACTOR_IS_MAPPED (stage_x11))
+          if (!STAGE_X11_IS_MAPPED (stage_x11))
             {
-              /* FIXME: This wont work if we support more states */
-              XChangeProperty (backend_x11->xdpy,
-                               stage_x11->xwin,
-                               backend_x11->atom_NET_WM_STATE, XA_ATOM, 32,
-                               PropModeReplace,
-                               (unsigned char *) &backend_x11->atom_NET_WM_STATE_FULLSCREEN, 1);
+              CLUTTER_NOTE (BACKEND, "Fullscreening unmapped stage");
+
+              update_state (stage_x11, backend_x11,
+                            &backend_x11->atom_NET_WM_STATE_FULLSCREEN,
+                            TRUE);
             }
           else
             {
+              CLUTTER_NOTE (BACKEND, "Fullscreening mapped stage");
+
               /* We need to fix the window size so that it will remove
                  the maximum and minimum window hints. Otherwise
                  metacity will honour the restrictions and not
@@ -447,20 +471,22 @@ clutter_stage_x11_set_fullscreen (ClutterStageWindow *stage_window,
     }
   else
     {
-      if (!STAGE_X11_IS_MAPPED (stage_x11))
-        stage_x11->fullscreen_on_map = FALSE;
+      stage_x11->fullscreening = FALSE;
 
       if (stage_x11->xwin != None)
         {
-          if (!CLUTTER_ACTOR_IS_MAPPED (stage_x11))
+          if (!STAGE_X11_IS_MAPPED (stage_x11))
             {
-              /* FIXME: This wont work if we support more states */
-              XDeleteProperty (backend_x11->xdpy,
-                               stage_x11->xwin, 
-                               backend_x11->atom_NET_WM_STATE);
+              CLUTTER_NOTE (BACKEND, "Un-fullscreening unmapped stage");
+
+              update_state (stage_x11, backend_x11,
+                            &backend_x11->atom_NET_WM_STATE_FULLSCREEN,
+                            FALSE);
             }
           else
             {
+              CLUTTER_NOTE (BACKEND, "Un-fullscreening mapped stage");
+
               send_wmspec_change_state (backend_x11,
                                         stage_x11->xwin,
                                         backend_x11->atom_NET_WM_STATE_FULLSCREEN,
@@ -468,13 +494,17 @@ clutter_stage_x11_set_fullscreen (ClutterStageWindow *stage_window,
 
               /* Fix the window size to restore the minimum/maximum
                  restriction */
-              clutter_stage_x11_fix_window_size (stage_x11, -1, -1);
+              clutter_stage_x11_fix_window_size (stage_x11,
+                                                 stage_x11->xwin_width,
+                                                 stage_x11->xwin_height);
             }
         }
     }
 
-  clutter_stage_ensure_viewport (CLUTTER_STAGE (stage_x11->wrapper));
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (stage_x11->wrapper));
+  /* XXX: Note we rely on the ConfigureNotify mechanism as the common
+   * mechanism to handle notifications of new X window sizes from the
+   * X server so we don't actively change the stage viewport here or
+   * queue a relayout etc. */
 }
 
 static void
@@ -504,7 +534,9 @@ clutter_stage_x11_set_user_resizable (ClutterStageWindow *stage_window,
 {
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
 
-  clutter_stage_x11_fix_window_size (stage_x11, -1, -1);
+  clutter_stage_x11_fix_window_size (stage_x11,
+                                     stage_x11->xwin_width,
+                                     stage_x11->xwin_height);
 }
 
 static void
@@ -575,14 +607,14 @@ clutter_stage_x11_show (ClutterStageWindow *stage_window,
           CLUTTER_NOTE (BACKEND, "Mapping stage[%lu]",
                         (unsigned long) stage_x11->xwin);
 
-          if (stage_x11->fullscreen_on_map)
-            clutter_stage_x11_set_fullscreen (stage_window, TRUE);
-          else
-            clutter_stage_x11_set_fullscreen (stage_window, FALSE);
-
           set_stage_state (stage_x11, STAGE_X11_WITHDRAWN, 0);
 
           update_wm_hints (stage_x11);
+
+          if (stage_x11->fullscreening)
+            clutter_stage_x11_set_fullscreen (stage_window, TRUE);
+          else
+            clutter_stage_x11_set_fullscreen (stage_window, FALSE);
         }
 
       g_assert (STAGE_X11_IS_MAPPED (stage_x11));
@@ -659,15 +691,12 @@ clutter_stage_x11_init (ClutterStageX11 *stage)
   stage->wm_state = STAGE_X11_WITHDRAWN;
 
   stage->is_foreign_xwin = FALSE;
-  stage->fullscreen_on_map = FALSE;
+  stage->fullscreening = FALSE;
   stage->is_cursor_visible = TRUE;
-  stage->viewport_initialized = FALSE;
 
   stage->title = NULL;
 
   stage->wrapper = NULL;
-
-  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_IS_TOPLEVEL);
 }
 
 static void
@@ -845,14 +874,13 @@ clutter_x11_set_stage_foreign (ClutterStage *stage,
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+  g_return_val_if_fail (!CLUTTER_ACTOR_IN_DESTRUCTION (stage), FALSE);
   g_return_val_if_fail (xwindow != None, FALSE);
 
   actor = CLUTTER_ACTOR (stage);
 
-  impl = _clutter_stage_get_window (stage);
-  stage_x11 = CLUTTER_STAGE_X11 (impl);
-
   xvisinfo = clutter_backend_x11_get_visual_info (backend_x11);
+  g_return_val_if_fail (xvisinfo != NULL, FALSE);
 
   clutter_x11_trap_x_errors ();
 
@@ -863,15 +891,32 @@ clutter_x11_set_stage_foreign (ClutterStage *stage,
                          &border,
                          &depth);
 
-  if (clutter_x11_untrap_x_errors () ||
-      !status ||
-      width == 0 || height == 0 ||
-      depth != xvisinfo->depth)
+  if (clutter_x11_untrap_x_errors () || !status)
     {
-      g_warning ("Unable to retrieve the new window geometry");
+      g_critical ("Unable to retrieve the geometry of the foreign window: "
+                  "XGetGeometry() failed (status code: %d)", status);
       return FALSE;
     }
 
+  if (width == 0 || height == 0)
+    {
+      g_warning ("The size of the foreign window is 0x0");
+      return FALSE;
+    }
+
+  if (depth != xvisinfo->depth)
+    {
+      g_warning ("The depth of the visual of the foreign window is %d, but "
+                 "Clutter has been initialized to require a visual depth "
+                 "of %d",
+                 depth,
+                 xvisinfo->depth);
+      return FALSE;
+    }
+
+  impl = _clutter_stage_get_window (stage);
+  stage_x11 = CLUTTER_STAGE_X11 (impl);
+
   fwd.stage_x11 = stage_x11;
   fwd.xwindow = xwindow;
 
@@ -890,7 +935,16 @@ clutter_x11_set_stage_foreign (ClutterStage *stage,
                             set_foreign_window_callback,
                             &fwd);
 
-  clutter_stage_ensure_viewport (stage);
+  /* Queue a relayout - so the stage will be allocated the new
+   * window size.
+   *
+   * Note also that when the stage gets allocated the new
+   * window size that will result in the stage's
+   * priv->viewport being changed, which will in turn result
+   * in the Cogl viewport changing when _clutter_do_redraw
+   * calls _clutter_stage_maybe_setup_viewport().
+   */
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (stage));
 
   return TRUE;
 }
