@@ -26,6 +26,7 @@
 
 /**
  * SECTION:clutter-x11-texture-pixmap
+ * @Title: ClutterX11TexturePixmap
  * @short_description: A texture which displays the content of an X Pixmap.
  *
  * #ClutterX11TexturePixmap is a class for displaying the content of an
@@ -303,6 +304,13 @@ free_damage_resources (ClutterX11TexturePixmap *texture)
     }
 }
 
+static gboolean
+clutter_x11_texture_pixmap_get_paint_volume (ClutterActor       *self,
+                                             ClutterPaintVolume *volume)
+{
+  return clutter_paint_volume_set_from_allocation (volume, self);
+}
+
 static void
 clutter_x11_texture_pixmap_real_queue_damage_redraw (
                                               ClutterX11TexturePixmap *texture,
@@ -311,13 +319,13 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
                                               gint width,
                                               gint height)
 {
+  ClutterX11TexturePixmapPrivate *priv = texture->priv;
   ClutterActor    *self = CLUTTER_ACTOR (texture);
   ClutterActorBox  allocation;
-  guint	           pixmap_width = 0;
-  guint            pixmap_height = 0;
   float            scale_x;
   float            scale_y;
-  ClutterActorBox  clip;
+  ClutterVertex    origin;
+  ClutterPaintVolume clip;
 
   /* NB: clutter_actor_queue_clipped_redraw expects a box in the actor's
    * coordinate space so we need to convert from pixmap coordinates to
@@ -336,24 +344,25 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
       return;
     }
 
+  if (priv->pixmap_width == 0 || priv->pixmap_height == 0)
+    return;
+
   clutter_actor_get_allocation_box (self, &allocation);
 
-  g_object_get (self,
-                "pixmap-width",  &pixmap_width,
-                "pixmap-height", &pixmap_height,
-                NULL);
+  scale_x = (allocation.x2 - allocation.x1) / priv->pixmap_width;
+  scale_y = (allocation.y2 - allocation.y1) / priv->pixmap_height;
 
-  scale_x = (allocation.x2 - allocation.x1) / pixmap_width;
-  scale_y = (allocation.y2 - allocation.y1) / pixmap_height;
+  _clutter_paint_volume_init_static (self, &clip);
 
-  clip.x1 = x * scale_x;
-  clip.y1 = y * scale_y;
-  clip.x2 =  clip.x1 + width * scale_x;
-  clip.y2 = clip.y1 + height * scale_y;
+  origin.x = x * scale_x;
+  origin.y = y * scale_y;
+  origin.z = 0;
+  clutter_paint_volume_set_origin (&clip, &origin);
+  clutter_paint_volume_set_width (&clip, width * scale_x);
+  clutter_paint_volume_set_height (&clip, height * scale_y);
 
-  _clutter_actor_queue_redraw_with_clip (self,
-                                         CLUTTER_REDRAW_CLIPPED_TO_BOX,
-                                         &clip);
+  _clutter_actor_queue_redraw_with_clip (self, 0, &clip);
+  clutter_paint_volume_free (&clip);
 }
 
 static void
@@ -389,17 +398,11 @@ static void
 clutter_x11_texture_pixmap_dispose (GObject *object)
 {
   ClutterX11TexturePixmap *texture = CLUTTER_X11_TEXTURE_PIXMAP (object);
-  ClutterX11TexturePixmapPrivate *priv = texture->priv;
 
   free_damage_resources (texture);
 
   clutter_x11_remove_filter (on_x_event_filter_too, (gpointer)texture);
-
-  if (priv->owns_pixmap && priv->pixmap)
-    {
-      XFreePixmap (clutter_x11_get_default_display (), priv->pixmap);
-      priv->pixmap = None;
-    }
+  clutter_x11_texture_pixmap_set_pixmap (texture, None);
 
   G_OBJECT_CLASS (clutter_x11_texture_pixmap_parent_class)->dispose (object);
 }
@@ -503,11 +506,14 @@ static void
 clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
 {
   GObjectClass      *object_class = G_OBJECT_CLASS (klass);
+  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   GParamSpec        *pspec;
   ClutterBackend    *default_backend;
 
   g_type_class_add_private (klass, sizeof (ClutterX11TexturePixmapPrivate));
 
+  actor_class->get_paint_volume = clutter_x11_texture_pixmap_get_paint_volume;
+
   object_class->dispose      = clutter_x11_texture_pixmap_dispose;
   object_class->set_property = clutter_x11_texture_pixmap_set_property;
   object_class->get_property = clutter_x11_texture_pixmap_get_property;
@@ -805,29 +811,34 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
 
   priv = texture->priv;
 
-  clutter_x11_trap_x_errors ();
-
   /* Get rid of the existing Cogl texture early because it may try to
      use the pixmap which we might destroy */
   material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (texture));
   if (material)
     cogl_material_set_layer (material, 0, COGL_INVALID_HANDLE);
 
-  status = XGetGeometry (clutter_x11_get_default_display(),
-                         (Drawable)pixmap,
-                         &root,
-                         &x,
-                         &y,
-                         &width,
-                         &height,
-                         &border_width,
-                         &depth);
+  if (pixmap != None)
+    {
+      clutter_x11_trap_x_errors ();
+      status = XGetGeometry (clutter_x11_get_default_display(),
+			     (Drawable)pixmap,
+			     &root,
+			     &x,
+			     &y,
+			     &width,
+			     &height,
+			     &border_width,
+			     &depth);
 
-  if (clutter_x11_untrap_x_errors () || status == 0)
+      if (clutter_x11_untrap_x_errors () || status == 0)
+	{
+	  g_warning ("Unable to query pixmap: %lx", pixmap);
+	  pixmap = None;
+	  width = height = depth = 0;
+	}
+    }
+  else
     {
-      if (pixmap != None)
-        g_warning ("Unable to query pixmap: %lx", pixmap);
-      pixmap = None;
       width = height = depth = 0;
     }
 
@@ -842,7 +853,7 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
       /* The damage object is created on the pixmap, so it needs to be
        * recreated with a change in pixmap.
        */
-      if (priv->automatic_updates && new_pixmap)
+      if (priv->automatic_updates)
 	{
 	  free_damage_resources (texture);
 	  create_damage_resources (texture);
@@ -871,6 +882,7 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
    * above members so the values are all available to the
    * signal handlers. */
   g_object_ref (texture);
+
   if (new_pixmap)
     g_object_notify (G_OBJECT (texture), "pixmap");
   if (new_pixmap_width)
@@ -941,6 +953,8 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
                                   CompositeRedirectAutomatic : CompositeRedirectManual);
       XSync (clutter_x11_get_default_display (), False);
       clutter_x11_untrap_x_errors ();
+
+      clutter_x11_texture_pixmap_set_pixmap (texture, None);
     }
 
   priv->window = window;
