@@ -342,7 +342,7 @@ test_script_layout_property (TestConformSimpleFixture *fixture,
                              gconstpointer dummy G_GNUC_UNUSED)
 {
   ClutterScript *script = clutter_script_new ();
-  GObject *manager, *container, *actor;
+  GObject *manager, *container, *actor1, *actor2;
   GError *error = NULL;
   gchar *test_file;
   gboolean x_fill, expand;
@@ -359,23 +359,25 @@ test_script_layout_property (TestConformSimpleFixture *fixture,
   g_assert (error == NULL);
 #endif
 
-  manager = container = actor = NULL;
+  manager = container = actor1 = actor2 = NULL;
   clutter_script_get_objects (script,
                               "manager", &manager,
                               "container", &container,
-                              "actor", &actor,
+                              "actor-1", &actor1,
+                              "actor-2", &actor2,
                               NULL);
 
   g_assert (CLUTTER_IS_LAYOUT_MANAGER (manager));
   g_assert (CLUTTER_IS_CONTAINER (container));
-  g_assert (CLUTTER_IS_ACTOR (actor));
+  g_assert (CLUTTER_IS_ACTOR (actor1));
+  g_assert (CLUTTER_IS_ACTOR (actor2));
 
   x_fill = FALSE;
   y_align = CLUTTER_BOX_ALIGNMENT_START;
   expand = FALSE;
   clutter_layout_manager_child_get (CLUTTER_LAYOUT_MANAGER (manager),
                                     CLUTTER_CONTAINER (container),
-                                    CLUTTER_ACTOR (actor),
+                                    CLUTTER_ACTOR (actor1),
                                     "x-fill", &x_fill,
                                     "y-align", &y_align,
                                     "expand", &expand,
@@ -385,5 +387,20 @@ test_script_layout_property (TestConformSimpleFixture *fixture,
   g_assert (y_align == CLUTTER_BOX_ALIGNMENT_CENTER);
   g_assert (expand);
 
+  x_fill = TRUE;
+  y_align = CLUTTER_BOX_ALIGNMENT_START;
+  expand = TRUE;
+  clutter_layout_manager_child_get (CLUTTER_LAYOUT_MANAGER (manager),
+                                    CLUTTER_CONTAINER (container),
+                                    CLUTTER_ACTOR (actor2),
+                                    "x-fill", &x_fill,
+                                    "y-align", &y_align,
+                                    "expand", &expand,
+                                    NULL);
+
+  g_assert (x_fill == FALSE);
+  g_assert (y_align == CLUTTER_BOX_ALIGNMENT_END);
+  g_assert (expand == FALSE);
+
   g_object_unref (script);
 }
